// VRML Generator
// Copyright Justin Couch 1996
//
// Chapter 13: Late Night VRML 2.0 and Java
//
// Anchor class

package vlc.vrml.external.geometry;

import java.io.*;
import java.util.*;
import vrml.external.*;
import vrml.external.field.*;

public class Anchor extends GroupingNode
{
	private EventInSFString	_set_description;
	private EventInMFString	_set_parameter;
	private EventInMFString	_set_url;

	private int num_children = 0;

	private String	_description;
	private String[]	_parameter;
	private String[]	_url;

	// constructor builds a copy of itself and is added to the scene
	// graph.
	public Anchor()
	{
		Browser b = Browser.getBrowser();

		if(b == null)
			return;

		node = b.createVrmlFromString("Anchor {}")[0];

		createFields();
	}

	public Anchor(Browser b)
	{
		if(b == null)
			return;

		node = b.createVrmlFromString("Anchor {}")[0];

		createFields();
	}

	public Anchor(float[] bboxCenter, float[] bboxSize)
	{
		Browser b = Browser.getBrowser();

		if((bboxCenter != null) && (bboxCenter.length > 2))
			_bboxCenter = bboxCenter;

		if((bboxSize != null) && (bboxSize.length > 2))
			_bboxSize = bboxSize;

		if(b == null)
			return;

		node = b.createVrmlFromString(createString())[0];

		createFields();
	}

	public Anchor(Browser b, float[] bboxCenter, float[] bboxSize)
	{
		if((bboxCenter != null) && (bboxCenter.length > 2))
			_bboxCenter = bboxCenter;

		if((bboxSize != null) && (bboxSize.length > 2))
			_bboxSize = bboxSize;

		if(b == null)
			return;

		node = b.createVrmlFromString(createString())[0];

		createFields();
	}

	private void createFields()
	{
		_addChildren = (EventInMFNode)node.getEventIn("addChildren");
		_removeChildren = (EventInMFNode)node.getEventIn("removeChildren");
		_set_children = (EventInMFNode)node.getEventIn("children");
		_set_description = (EventInSFString)node.getEventIn("description");
		_set_parameter = (EventInMFString)node.getEventIn("parameter");
		_set_url = (EventInMFString)node.getEventIn("url");

		have_browser = true;
		type = vlc.vrml.VrmlTypes.Anchor;
	}

	private String createString()
	{
		String vrml_string = "Anchor { ";

		if((_bboxCenter[0] != 0) || (_bboxCenter[1] != 0) ||
		   (_bboxCenter[2] != 0))
		{
			vrml_string += "bboxCenter " +
							_bboxCenter[0] + " " +
							_bboxCenter[1] + " " +
							_bboxCenter[2] + " ";
		}

		if((_bboxSize[0] != -1) || (_bboxSize[1] != -1) ||
		   (_bboxSize[2] != -1))
		{
			vrml_string += "bboxCenter " +
							_bboxSize[0] + " " +
							_bboxSize[1] + " " +
							_bboxSize[2] + " ";
		}

		vrml_string += " }";

		return(vrml_string);
	}

	public void cleanup()
	{
		int	i;

		// force the clean up of internally allocated objects
		for(i = 0; i < _children.size(); i++)
			removeChildren((ChildrenNode)_children.elementAt(i));
			
		_children = null;
		_bboxCenter = null;
		_bboxSize = null;
		
		_description = null;
		_parameter = null;
		_url = null;
	}

	public void writeToFile(PrintStream fp, int indent)
	{
		int	i;
		StringBuffer buffer = new StringBuffer();

		for(i = 0; i < indent; i++)
			buffer.append("  ");

		fp.print(buffer.toString());
		if(name != null)
			fp.print("DEF " + name + " ");

		fp.println("Anchor {");

		// print the contents of the node

		if(num_children != 0)
		{
			Enumeration e = _children.elements();

			fp.print(buffer.toString() + "  children " );
			if(num_children != 1)
				fp.println("[");

			for(;e.hasMoreElements();)
				((VrmlObject)(e.nextElement())).writeToFile(fp, indent + 2);

			if(num_children != 1)
				fp.println(buffer.toString() + "  ]");
		}

		if((_bboxCenter[0] != 0) || (_bboxCenter[1] != 0) ||
		   (_bboxCenter[2] != 0))
			fp.println(buffer.toString() + "  bboxCenter " +
						_bboxCenter[0] + " " +
						_bboxCenter[1] + " " +
						_bboxCenter[2]);

		if((_bboxSize[0] != -1) || (_bboxSize[1] != -1) ||
		   (_bboxSize[2] != -1))
			fp.println(buffer.toString() + "  bboxSize " +
						_bboxSize[0] + " " +
						_bboxSize[1] + " " +
						_bboxSize[2]);


		if(_description != null)

			fp.println(buffer.toString() + "  description " + _description);

			

		if(_parameter != null)

		{
			if(_parameter.length > 0)
			{
				fp.println(buffer.toString() + " parameter [");

				for(i = 0; i < _parameter.length; i++)
				{
					fp.println(buffer.toString() + "    \"" +
						   _parameter[i] + "\"");
				}
				
				fp.println(buffer.toString() + "  ]");
			}

			

		}

		
		if(_url.length > 0)
		{
			fp.println(buffer.toString() + "  url [");

			for(i = 0; i < _url.length; i++)
			{
				fp.println(buffer.toString() + "    \"" +
					   _url[i] + "\"");
			}
			fp.println(buffer.toString() + "  ]");
		}
		
		fp.println(buffer.toString() + "}");
	}

	public void set_description(String val)
	{
		_description = val;
		if(have_browser)
			_set_description.setValue(val);
	}

	public void set_parameter(String[] val)
	{
		_parameter = val;
		if(have_browser)
			_set_parameter.setValue(val);
	}

	public void set_url(String[] val)
	{
		_url = val;
		if(have_browser)
			_set_url.setValue(val);
	}
	
	public String get_description()
	{
		return(_description);
	}

	public String[] get_parameter()
	{
		return(_parameter);
	}

	public String[] get_url()
	{
		return(_url);
	}
}
