// VRML Generator
// Copyright Justin Couch 1996
//
// Chapter 13: Late Night VRML 2.0 and Java
//
// AudioClip class

package vlc.vrml.external.geometry;

import java.io.PrintStream;
import vrml.external.*;
import vrml.external.field.*;
import vlc.vrml.VrmlTypes;

public class AudioClip extends VrmlObject implements AudioNode
{
	private EventInMFString _set_url;
	private EventInSFBool	_set_loop;
	private EventInSFFloat	_set_pitch;
	private EventInSFTime	_set_starttime;
	private EventInSFTime	_set_stoptime;
	private EventInSFString	_set_description;

	private EventOutSFFloat	_get_duration_changed;
	private EventOutSFBool	_get_isActive;

	private String[]	_url;
	private boolean	_loop = false;
	private float	_pitch = 1;
	private double	_startTime = 0;
	private double	_stopTime = 0;
	private String	_description = null;

	// Null constructor since you cannot create just a straight texture
	public AudioClip()
	{
		Browser b = Browser.getBrowser();

		if(b == null)
			return;

		node = b.createVrmlFromString("AudioClip {}")[0];

		createFields();
	}

	public AudioClip(Browser b)
	{
		if(b == null)
			return;

		node = b.createVrmlFromString("AudioClip {}")[0];

		createFields();
	}

	public void createFields()
	{
		_set_url = (EventInMFString)node.getEventIn("url");
		_set_loop = (EventInSFBool)node.getEventIn("loop");
		_set_pitch = (EventInSFFloat)node.getEventIn("pitch");
		_set_starttime = (EventInSFTime)node.getEventIn("startTime");
		_set_stoptime = (EventInSFTime)node.getEventIn("stopTime");
		_set_description = (EventInSFString)node.getEventIn("description");

		_get_duration_changed = (EventOutSFFloat)node.getEventOut("duration_changed");
		_get_isActive = (EventOutSFBool)node.getEventOut("isActive");

		have_browser = true;
		type = VrmlTypes.AudioClip;
	}

	public void writeToFile(PrintStream fp, int indent)
	{
		int     i;
		StringBuffer buffer = new StringBuffer();

		for(i = 0; i < indent; i++)
			buffer.append("  ");

		if(name != null)
			fp.print("DEF " + name + " ");

		fp.println("AudioClip {");

		if(_url.length > 0)
		{
			fp.println(buffer.toString() + "  _url [");

			for(i = 0; i < _url.length; i++)
			{
				fp.println(buffer.toString() + "    \"" +
					   _url[i] + "\"");
			}
			fp.println(buffer.toString() + " ]");
		}

		if(_loop)
			fp.println(buffer.toString() + "  loop TRUE");
		if(_pitch != 1)
			fp.println(buffer.toString() + "  pitch " + _pitch);
		if(_startTime != 0)
			fp.println(buffer.toString() + "  startTime " + _startTime);
		if(_stopTime != 0)
			fp.println(buffer.toString() + "  stopTime " + _stopTime);
		if(_description != null)
			fp.println(buffer.toString() + "  desciption \"" + _description + "\"");
	}

	public void set_url(String[] url)
	{
		_url = url;
		if(have_browser)
			_set_url.setValue(_url);
	}

	public String[] get_url()
	{
		return(_url);
	}

	public void set_loop(boolean val)
	{
		_loop = val;
		if(have_browser)
			_set_loop.setValue(val);
	}

	public boolean get_loop()
	{
		return(_loop);
	}

	public void set_pitch(float val)
	{
		_pitch = val;

		if(have_browser)
			_set_pitch.setValue(val);
	}

	public float get_pitch()
	{
		return(_pitch);
	}

	public void set_startTime(double val)
	{
		_startTime = val;

		if(have_browser)
			_set_starttime.setValue(val);
	}

	public double get_startTime()
	{
		return(_startTime);
	}

	public void set_stopTime(double val)
	{
		_stopTime = val;

		if(have_browser)
			_set_stoptime.setValue(val);
	}

	public double get_stopTime()
	{
		return(_stopTime);
	}

	public void set_description(String description)
	{
		_description = description;
		if(have_browser)
			_set_description.setValue(description);
	}

	public String get_description()
	{
		return(_description);
	}

	public float get_duration_changed()
	{
		return(_get_duration_changed.getValue());
	}

	public boolean get_isActive()
	{
		return(_get_isActive.getValue());
	}
}
