// VRML Generator
// Copyright Justin Couch 1996
//
// Chapter 13: Late Night VRML 2.0 and Java
//
// Billboard class

package vlc.vrml.external.geometry;

import java.io.*;
import java.util.*;
import vrml.external.*;
import vrml.external.field.*;

public class Billboard extends GroupingNode
{
	private EventInSFVec3f	_set_axisOfRotation;

	private int num_children = 0;

	private float[] _axis = { 0, 1, 0 };

	// constructor builds a copy of itself and is added to the scene
	// graph.
	public Billboard()
	{
		Browser b = Browser.getBrowser();

		if(b == null)
			return;

		node = b.createVrmlFromString("Billboard {}")[0];

		createFields();
	}

	public Billboard(Browser b)
	{
		if(b == null)
			return;

		node = b.createVrmlFromString("Billboard {}")[0];

		createFields();
	}

	public Billboard(float[] bboxCenter, float[] bboxSize)
	{
		Browser b = Browser.getBrowser();

		if((bboxCenter != null) && (bboxCenter.length > 2))
			_bboxCenter = bboxCenter;

		if((bboxSize != null) && (bboxSize.length > 2))
			_bboxSize = bboxSize;

		if(b == null)
			return;

		node = b.createVrmlFromString(createString())[0];

		createFields();
	}

	public Billboard(Browser b, float[] bboxCenter, float[] bboxSize)
	{
		if((bboxCenter != null) && (bboxCenter.length > 2))
			_bboxCenter = bboxCenter;

		if((bboxSize != null) && (bboxSize.length > 2))
			_bboxSize = bboxSize;

		if(b == null)
			return;

		node = b.createVrmlFromString(createString())[0];

		createFields();
	}

	private void createFields()
	{
		_addChildren = (EventInMFNode)node.getEventIn("addChildren");
		_removeChildren = (EventInMFNode)node.getEventIn("removeChildren");
		_set_children = (EventInMFNode)node.getEventIn("children");

		_set_axisOfRotation = (EventInSFVec3f)node.getEventIn("axisOfRotation");

		have_browser = true;
		type = vlc.vrml.VrmlTypes.Billboard;
	}

	private String createString()
	{
		String vrml_string = "Billboard { ";

		if((_bboxCenter[0] != 0) || (_bboxCenter[1] != 0) ||
		   (_bboxCenter[2] != 0))
		{
			vrml_string += "bboxCenter " +
							_bboxCenter[0] + " " +
							_bboxCenter[1] + " " +
							_bboxCenter[2] + " ";
		}

		if((_bboxSize[0] != -1) || (_bboxSize[1] != -1) ||
		   (_bboxSize[2] != -1))
		{
			vrml_string += "bboxCenter " +
							_bboxSize[0] + " " +
							_bboxSize[1] + " " +
							_bboxSize[2] + " ";
		}

		vrml_string += " }";

		return(vrml_string);
	}

	public void cleanup()
	{
		int	i;

		for(i = 0; i < _children.size(); i++)
			removeChildren((ChildrenNode)_children.elementAt(i));

		// force the clean up of internally allocated objects
		_children = null;
		_bboxCenter = null;
		_bboxSize = null;
	}

	public void writeToFile(PrintStream fp, int indent)
	{
		int	i;
		StringBuffer buffer = new StringBuffer();

		for(i = 0; i < indent; i++)
			buffer.append("  ");

		fp.print(buffer.toString());
		if(name != null)
			fp.print("DEF " + name + " ");

		fp.println("Billboard {");

		// print the contents of the node

		if(num_children != 0)
		{
			Enumeration e = _children.elements();

			fp.print(buffer.toString() + "  children " );
			if(num_children != 1)
				fp.println("[");

			for(;e.hasMoreElements();)
				((VrmlObject)(e.nextElement())).writeToFile(fp, indent + 2);

			if(num_children != 1)
				fp.println(buffer.toString() + "  ]");
		}

		if((_bboxCenter[0] != -1) || (_bboxCenter[1] != -1) ||
		   (_bboxCenter[2] != -1))
			fp.println(buffer.toString() + " bboxCenter " +
						_bboxCenter[0] + " " +
						_bboxCenter[1] + " " +
						_bboxCenter[2]);

		if((_bboxSize[0] != -1) || (_bboxSize[1] != -1) ||
		   (_bboxSize[2] != -1))
			fp.println(buffer.toString() + " bboxSize " +
						_bboxSize[0] + " " +
						_bboxSize[1] + " " +
						_bboxSize[2]);

		fp.println(buffer.toString() + "}");
	}

	public void set_axisOfRotation(float[] val)
	{
		_axis = val;
		if(have_browser)
			_set_axisOfRotation.setValue(val);
	}

	public float[] get_axisOfRotation()
	{
		return(_axis);
	}
}
