// VRML Generator
// Copyright Justin Couch 1996
//
// Chapter 13: Late Night VRML 2.0 and Java
//
// Collision class
// The class does not support proxy nodes yet. Still trying to work this out.

package vlc.vrml.external.geometry;

import java.io.*;
import java.util.*;
import vrml.external.*;
import vrml.external.field.*;

public class Collision extends GroupingNode
{
	private EventInSFBool	_set_collide;
	private EventOutSFTime	_get_collideTime;

	private int num_children = 0;
	private boolean _collide = true;

	// constructor builds a copy of itself and is added to the scene
	// graph.
	public Collision()
	{
		Browser b = Browser.getBrowser();

		if(b == null)
			return;

		node = b.createVrmlFromString("Collision {}")[0];

		createFields();
	}

	public Collision(Browser b)
	{
		if(b == null)
			return;

		node = b.createVrmlFromString("Collision {}")[0];

		createFields();
	}

	public Collision(float[] bboxCenter, float[] bboxSize)
	{
		Browser b = Browser.getBrowser();

		if((bboxCenter != null) && (bboxCenter.length > 2))
			_bboxCenter = bboxCenter;

		if((bboxSize != null) && (bboxSize.length > 2))
			_bboxSize = bboxSize;

		if(b == null)
			return;

		node = b.createVrmlFromString(createString())[0];

		createFields();
	}

	public Collision(Browser b, float[] bboxCenter, float[] bboxSize)
	{
		if((bboxCenter != null) && (bboxCenter.length > 2))
			_bboxCenter = bboxCenter;

		if((bboxSize != null) && (bboxSize.length > 2))
			_bboxSize = bboxSize;

		if(b == null)
			return;

		node = b.createVrmlFromString(createString())[0];

		createFields();
	}

	private void createFields()
	{
		_addChildren = (EventInMFNode)node.getEventIn("addChildren");
		_removeChildren = (EventInMFNode)node.getEventIn("removeChildren");
		_set_children = (EventInMFNode)node.getEventIn("children");

		_set_collide = (EventInSFBool)node.getEventIn("collide");
		_get_collideTime = (EventOutSFTime)node.getEventOut("collideTime");

		have_browser = true;
		type = vlc.vrml.VrmlTypes.Collision;
	}

	private String createString()
	{
		String vrml_string = "Collision { ";

		if((_bboxCenter[0] != 0) || (_bboxCenter[1] != 0) ||
		   (_bboxCenter[2] != 0))
		{
			vrml_string += "bboxCenter " +
							_bboxCenter[0] + " " +
							_bboxCenter[1] + " " +
							_bboxCenter[2] + " ";
		}

		if((_bboxSize[0] != -1) || (_bboxSize[1] != -1) ||
		   (_bboxSize[2] != -1))
		{
			vrml_string += "bboxCenter " +
							_bboxSize[0] + " " +
							_bboxSize[1] + " " +
							_bboxSize[2] + " ";
		}

		vrml_string += " }";

		return(vrml_string);
	}

	public void cleanup()
	{
		int	i;

		for(i = 0; i < _children.size(); i++)
			removeChildren((ChildrenNode)_children.elementAt(i));

		// force the clean up of internally allocated objects
		_children = null;
		_bboxCenter = null;
		_bboxSize = null;
	}

	public void writeToFile(PrintStream fp, int indent)
	{
		int	i;
		StringBuffer buffer = new StringBuffer();

		for(i = 0; i < indent; i++)
			buffer.append("  ");

		fp.print(buffer.toString());
		if(name != null)
			fp.print("DEF " + name + " ");

		fp.println("Collision {");

		// print the contents of the node

		if(num_children != 0)
		{
			Enumeration e = _children.elements();

			fp.print(buffer.toString() + "  children " );
			if(num_children != 1)
				fp.println("[");

			for(;e.hasMoreElements();)
				((VrmlObject)(e.nextElement())).writeToFile(fp, indent + 2);

			if(num_children != 1)
				fp.println(buffer.toString() + "  ]");
		}

		if((_bboxCenter[0] != 0) || (_bboxCenter[1] != 0) ||
		   (_bboxCenter[2] != 0))
			fp.println(buffer.toString() + "  bboxCenter " +
						_bboxCenter[0] + " " +
						_bboxCenter[1] + " " +
						_bboxCenter[2]);

		if((_bboxSize[0] != -1) || (_bboxSize[1] != -1) ||
		   (_bboxSize[2] != -1))
			fp.println(buffer.toString() + "  bboxSize " +
						_bboxSize[0] + " " +
						_bboxSize[1] + " " +
						_bboxSize[2]);


		if(!_collide)

			fp.println(buffer.toString() + "  collide FALSE");

		fp.println(buffer.toString() + "}");
	}

	public void set_collide(boolean val)
	{
		_collide = val;
		if(have_browser)
			_set_collide.setValue(val);
	}

	public boolean get_collide()
	{
		return(_collide);
	}

	public double get_collideTime()
	{
		return(_get_collideTime.getValue());
	}
}
