// VRML Generator
// Copyright Justin Couch 1996
//
// Chapter 13: Late Night VRML 2.0 and Java
//
// Directional Light class

package vlc.vrml.external.geometry;

import java.io.*;
import vrml.external.*;
import vrml.external.field.*;

public class DirectionalLight extends Light
{
	private EventInSFVec3f	_set_direction;

	private float[]	_direction = { 0, 0, -1};

	public DirectionalLight()
	{
		Browser b = Browser.getBrowser();

		if(b == null)
			return;

		createFields();
	}

	public DirectionalLight(Browser b)
	{
		if(b == null)
			return;

		node = b.createVrmlFromString("DirectionalLight {}")[0];

		createFields();
	}

	private void createFields()
	{
		_set_on = (EventInSFBool)node.getEventIn("on");
		_set_intensity = (EventInSFFloat)node.getEventIn("intensity");
		_set_ambientIntensity = (EventInSFFloat)node.getEventIn("ambientIntensity");
		_set_color = (EventInSFColor)node.getEventIn("color");
		_set_direction = (EventInSFVec3f)node.getEventIn("direction");

		have_browser = true;
		type = vlc.vrml.VrmlTypes.DirectionalLight;
	}

	public void writeToFile(PrintStream fp, int indent)
	{
		int	i;
		StringBuffer buffer = new StringBuffer();

		for(i = 0; i < indent; i++)
			buffer.append("  ");

		if(name != null)
			fp.print("DEF " + name + " ");

		fp.println(buffer.toString() + "DirectionalLight {");

		if(!_on)
			fp.println(buffer.toString() + "  on FALSE");

		if(_ambientIntensity != 0)
			fp.println(buffer.toString() +
					   "  ambientIntensity " +
					   _ambientIntensity);

		if(_intensity != 0)
			fp.println(buffer.toString() +
					   "  intensity " +
					   _intensity);

		if((_color[0] != 1) || (_color[1] != 1) || (_color[2] != 1))
			fp.println(buffer.toString() + "  color "
					   + _color[0] + " " +
					   + _color[1] + " " +
					   + _color[2]);

		if((_direction[0] != 0) || (_direction[1] != 0) ||
		   (_direction[2] != -1))
			fp.println(buffer.toString() + "  direction "
					   + _direction[0] + " " +
					   + _direction[1] + " " +
					   + _direction[2]);

		fp.println(buffer.toString() + "}");
	}

	public void set_direction(float[] val)
	{
		_direction = val;
		if(have_browser)
			_set_direction.setValue(val);
	}

	public float[] get_direction()
	{
		return(_direction);
	}
}