// VRML Generator
// Copyright Justin Couch 1996
//
// Chapter 13: Late Night VRML 2.0 and Java
//
// Inline class

package vlc.vrml.external.geometry;

import java.io.*;
import vrml.external.*;
import vrml.external.field.*;

public class Inline extends SpecialGroup
{
	private EventInMFString	_set_url;

	private String[] _url;
	private float[] _bboxCenter = { 0, 0, 0};
	private float[] _bboxSize = {-1, -1, -1};

	public Inline()
	{
		Browser b = Browser.getBrowser();

		if(b == null)
			return;

		node = b.createVrmlFromString("Inline {}")[0];

		createFields();
	}

	public Inline(Browser b)
	{
		if(b == null)
			return;

		node = b.createVrmlFromString("Inline {}")[0];

		createFields();
	}

	public Inline(float[] bboxCenter, float[] bboxSize)
	{
		Browser b = Browser.getBrowser();

		if((bboxCenter != null) && (bboxCenter.length > 2))
			_bboxCenter = bboxCenter;

		if((bboxSize != null) && (bboxSize.length > 2))
			_bboxSize = bboxSize;

		if(b == null)
			return;

		node = b.createVrmlFromString(createString())[0];

		createFields();
	}

	public Inline(Browser b, float[] bboxCenter, float[] bboxSize)
	{
		if(b == null)
			return;

		node = b.createVrmlFromString(createString())[0];

		createFields();
	}

	private void createFields()
	{
		_set_url = (EventInMFString)node.getEventIn("url");

		have_browser = true;
		type = vlc.vrml.VrmlTypes.Inline;
	}

	private String createString()
	{
		String vrml_string = "Inline { ";

		if((_bboxCenter[0] != 0) || (_bboxCenter[1] != 0) ||
		   (_bboxCenter[2] != 0))
		{
			vrml_string += "bboxCenter " +
							_bboxCenter[0] + " " +
							_bboxCenter[1] + " " +
							_bboxCenter[2] + " ";
		}

		if((_bboxSize[0] != -1) || (_bboxSize[1] != -1) ||
		   (_bboxSize[2] != -1))
		{
			vrml_string += "bboxCenter " +
							_bboxSize[0] + " " +
							_bboxSize[1] + " " +
							_bboxSize[2] + " ";
		}

		vrml_string += " }";

		return(vrml_string);
	}

	public void cleanup()
	{
		// force the clean up of internally allocated objects
		_url = null;
		_bboxCenter = null;
		_bboxSize = null;
	}

	public void writeToFile(PrintStream fp, int indent)
	{
		int	i;
		StringBuffer buffer = new StringBuffer();

		for(i = 0; i < indent; i++)
			buffer.append("  ");

		fp.print(buffer.toString());
		if(name != null)
			fp.print("DEF " + name + " ");

		fp.println("Inline {");

		// print the contents of the node

		if((_bboxCenter[0] != 0) || (_bboxCenter[1] != 0) ||
		   (_bboxCenter[2] != 0))
			fp.println(buffer.toString() + " bboxCenter " +
						_bboxCenter[0] + " " +
						_bboxCenter[1] + " " +
						_bboxCenter[2]);

		if((_bboxSize[0] != -1) || (_bboxSize[1] != -1) ||
		   (_bboxSize[2] != -1))
			fp.println(buffer.toString() + " bboxSize " +
						_bboxSize[0] + " " +
						_bboxSize[1] + " " +
						_bboxSize[2]);

		if(_url.length > 0)
		{
			fp.println(buffer.toString() + "  url [");

			for(i = 0; i < _url.length; i++)
			{
				fp.println(buffer.toString() + "    \"" +
					   _url[i] + "\"");
			}
			fp.println(buffer.toString() + "  ]");
		}

		fp.println(buffer.toString() + "}");
	}

	
	public void set_url(String[] val)
	{
		_url = val;
		if(have_browser)
			_set_url.setValue(val);
	}
	
	public String[] get_url()
	{
		return(_url);
	}
}
