// VRML Generator
// Copyright Justin Couch 1996
//
// Chapter 13: Late Night VRML 2.0 and Java
//
// MovieTexture class

package vlc.vrml.external.geometry;

import java.io.PrintStream;
import vrml.external.*;
import vrml.external.field.*;

public class MovieTexture extends Texture implements AudioNode
{
	private EventInMFString _set_url;
	private EventInSFBool	_set_loop;
	private EventInSFFloat	_set_speed;
	private EventInSFTime	_set_starttime;
	private EventInSFTime	_set_stoptime;

	private EventOutSFFloat	_get_duration_changed;
	private EventOutSFBool	_get_isActive;

	private String[]	_url;
	private boolean	loop = false;
	private float	speed = 1;
	private double	startTime = 0;
	private double	stopTime = 0;


	// Null constructor since you cannot create just a straight texture
	public MovieTexture()
	{
		Browser b = Browser.getBrowser();

		if(b == null)
			return;

		node = b.createVrmlFromString("MovieTexture {}")[0];

		createFields();
	}

	public MovieTexture(Browser b)
	{
		if(b == null)
			return;

		node = b.createVrmlFromString("MovieTexture {}")[0];

		createFields();
	}

	public MovieTexture(boolean rptS, boolean rptT)
	{
		String  tmp = "MovieTexture { ";

		repeatS = rptS;
		repeatT = rptT;

		if(!repeatS)
			tmp += "repeatS FALSE ";
		if(!repeatT)
			tmp += "repeatT FALSE ";

		tmp += "}";

		Browser b = Browser.getBrowser();

		if(b == null)
			return;

		node = b.createVrmlFromString(tmp)[0];

		createFields();
	}

	public MovieTexture(Browser b, boolean rptS, boolean rptT)
	{
		String  tmp = "MovieTexture { ";

		repeatS = rptS;
		repeatT = rptT;

		if(!repeatS)
			tmp += "repeatS FALSE ";
		if(!repeatT)
			tmp += "repeatT FALSE ";

		tmp += "}";

		if(b == null)
			return;

		node = b.createVrmlFromString(tmp)[0];

		createFields();
	}

	public void createFields()
	{
		_set_url = (EventInMFString)node.getEventIn("url");
		_set_loop = (EventInSFBool)node.getEventIn("loop");
		_set_speed = (EventInSFFloat)node.getEventIn("speed");
		_set_starttime = (EventInSFTime)node.getEventIn("startTime");
		_set_stoptime = (EventInSFTime)node.getEventIn("stopTime");

		_get_duration_changed = (EventOutSFFloat)node.getEventOut("duration_changed");
		_get_isActive = (EventOutSFBool)node.getEventOut("isActive");

		have_browser = true;
		type = vlc.vrml.VrmlTypes.MovieTexture;
	}

	public void finalize()
	{
		_url = null;
	}

	public void writeToFile(PrintStream fp, int indent)
	{
		int     i;
		StringBuffer buffer = new StringBuffer();

		for(i = 0; i < indent; i++)
			buffer.append("  ");

		fp.print(buffer.toString() + "texture ");
		if(name != null)
			fp.print("DEF " + name + " ");

		fp.println("MovieTexture {");

		if(!repeatS)
			fp.println(buffer.toString() + " repeatS FALSE");
		if(!repeatT)
			fp.println(buffer.toString() + " repeatT FALSE");

		if(_url.length > 0)
		{
			fp.println(buffer.toString() + " url [");

			for(i = 0; i < _url.length; i++)
			{
				fp.println(buffer.toString() + "  \"" +
					   _url[i] + "\"");
			}
			fp.println(buffer.toString() + " ]");
		}

		if(loop)
			fp.println(buffer.toString() + " loop TRUE");
		if(speed != 1)
			fp.println(buffer.toString() + " speed " + speed);
		if(startTime != 0)
			fp.println(buffer.toString() + " startTime " + startTime);
		if(stopTime != 0)
			fp.println(buffer.toString() + " stopTime " + stopTime);

	}

	public void set_url(String[] url)
	{
		_url = url;

		if(have_browser)
			_set_url.setValue(_url);
	}

	public String[] get_url()
	{
		return(_url);
	}

	public void set_loop(boolean val)
	{
		loop = val;

		if(have_browser)
			_set_loop.setValue(val);
	}

	public boolean get_loop()
	{
		return(loop);
	}

	public void set_speed(float val)
	{
		speed = val;

		if(have_browser)
			_set_speed.setValue(val);
	}

	public float get_speed()
	{
		return(speed);
	}

	public void set_startTime(double val)
	{
		startTime = val;

		if(have_browser)
			_set_starttime.setValue(val);
	}

	public double get_startTime()
	{
		return(startTime);
	}

	public void set_stopTime(double val)
	{
		stopTime = val;

		if(have_browser)
			_set_stoptime.setValue(val);
	}

	public double get_stopTime()
	{
		return(stopTime);
	}

	public float get_duration_changed()
	{
		return(_get_duration_changed.getValue());
	}

	public boolean get_isActive()
	{
		return(_get_isActive.getValue());
	}
}
