// VRML Generator
// Copyright Justin Couch 1996
//
// Chapter 13: Late Night VRML 2.0 and Java
//
// Script class.
// The extra fields are handled by the user passing in an array of strings.
// Each string is to consist of a single line which contains
//     access_type field_type field_name [field_value]

package vlc.vrml.external.geometry;

import java.io.PrintStream;
import vrml.external.*;
import vrml.external.field.*;

public class Script extends ChildrenNode
{
	EventInMFString _set_url;

	String[]	_url;
	boolean		_directOutput = false;
	boolean		_mustEvaluate = false;
	String[]	_fields = null;


	// this is the one case where the default constuctors are useless.
	// so don't put them in!

	public Script(boolean directOutput,
				  boolean mustEvaluate,
				  String[] field_list)
	{
		int	i;

		_directOutput = directOutput;
		_mustEvaluate = mustEvaluate;
		_fields = field_list;

		Browser b = Browser.getBrowser();

		if(b == null)
			return;

		node = b.createVrmlFromString(createString())[0];

		createFields();
	}

	public Script(Browser b,
				  boolean directOutput,
				  boolean mustEvaluate,
				  String[] field_list)
	{
		int	i;

		_directOutput = directOutput;
		_mustEvaluate = mustEvaluate;
		_fields = field_list;

		if(b == null)
			return;

		node = b.createVrmlFromString(createString())[0];

		createFields();
	}

	private void createFields()
	{
		_set_url = (EventInMFString)node.getEventIn("url");

		have_browser = true;
		type = vlc.vrml.VrmlTypes.Script;
	}

	private String createString()
	{
		int		i;
		String  vrml_string = "Script { ";

		if(!_directOutput)
			vrml_string += "directOutput FALSE ";
		if(!_mustEvaluate)
			vrml_string += "mustEvaluate FALSE ";

		for(i = 0; i < _fields.length; i++)
			vrml_string += _fields[i] + " ";

		vrml_string += "}";

		return(vrml_string);
	}

	public void cleanup()
	{
		_url = null;
	}

	public void writeToFile(PrintStream fp, int indent)
	{
		int     i;
		StringBuffer buffer = new StringBuffer();

		for(i = 0; i < indent; i++)
			buffer.append("  ");

		if(name != null)
			fp.print(buffer.toString() + "DEF " + name + " ");

		fp.println("Script {");

		if(!_directOutput)
			fp.println(buffer.toString() + "  _directOutput FALSE");
		if(!_mustEvaluate)
			fp.println(buffer.toString() + "  _mustEvaluate FALSE");

		if(_url.length > 0)
		{
			fp.println(buffer.toString() + " url [");

			for(i = 0; i < _url.length; i++)
			{
				fp.println(buffer.toString() + "    \"" +
					   _url[i] + "\"");
			}
			fp.println(buffer.toString() + "  ]");
		}

		if(_fields.length > 0)
		{
			for(i = 0; i < _fields.length; i++)
			{
				fp.println(buffer.toString() + "  " + _fields[i]);
			}
		}
		fp.println(buffer.toString() + "}");
	}

	public void set_url(String[] url)
	{
		_url = url;
		if(have_browser)
			_set_url.setValue(url);
	}

	public String[] get_url()
	{
		return(_url);
	}

	public boolean get_directOutput()
	{
		return(_directOutput);
	}

	public boolean get_mustEvaluate()
	{
		return(_mustEvaluate);
	}

	public String[] get_fields()
	{
		return(_fields);
	}
}
