// VRML Generator
// (c) Justin Couch 1996
//
// From Chapter 13 Late Night VRML 2.0 and Java
//
// Sound Class.

package vlc.vrml.external.geometry;

import java.io.PrintStream;
import java.util.*;
import vrml.external.*;
import vrml.external.field.*;
import vlc.vrml.VrmlTypes;

public class Sound extends ChildrenNode
{
	private EventInSFVec3f	_set_direction;
	private EventInSFFloat	_set_intensity;
	private EventInSFVec3f	_set_location;
	private EventInSFFloat	_set_maxBack;
	private EventInSFFloat	_set_maxFront;
	private EventInSFFloat	_set_minBack;
	private EventInSFFloat	_set_minFront;
	private EventInSFFloat	_set_priority;
	private EventInSFNode	_set_source;

	private float[]	_direction = { 0, 0, 1};
	private float	_intensity = 1;
	private float[]	_location = { 0, 0, 0};
	private float	_maxBack = 10;
	private float	_maxFront = 10;
	private float	_minBack = 1;
	private float	_minFront = 1;
	private float	_priority = 0;
	private VrmlObject	_source = null;	// an instance of AudioNode
	private boolean	_spatialize = true;

	public Sound()
	{
		Browser b = Browser.getBrowser();

		if(b == null)
			return;

		node = b.createVrmlFromString("Sound {}")[0];

		createFields();
	}

	public Sound(Browser b)
	{
		if(b == null)
			return;

		node = b.createVrmlFromString("Sound {}")[0];

		createFields();
	}

	public Sound(boolean spatialize)
	{
		String vrml_string;

		_spatialize = spatialize;

		Browser b = Browser.getBrowser();

		if(b == null)
			return;

		if(!spatialize)
			vrml_string = "Sound { spatialize FALSE }";
		else
			vrml_string = "Sound";

		node = b.createVrmlFromString(vrml_string)[0];

		createFields();
	}

	public Sound(Browser b, boolean spatialize)
	{
		String vrml_string;

		_spatialize = spatialize;

		if(b == null)
			return;

		if(!spatialize)
			vrml_string = "Sound { spatialize FALSE }";
		else
			vrml_string = "Sound";

		node = b.createVrmlFromString(vrml_string)[0];

		createFields();
	}

	private void createFields()
	{
		_set_direction = (EventInSFVec3f)node.getEventIn("direction");
		_set_intensity = (EventInSFFloat)node.getEventIn("intensity");
		_set_location = (EventInSFVec3f)node.getEventIn("location");
		_set_maxBack = ( EventInSFFloat)node.getEventIn("maxBack");
		_set_maxFront = ( EventInSFFloat)node.getEventIn("maxFront");
		_set_minBack = ( EventInSFFloat)node.getEventIn("minBack");
		_set_minFront = ( EventInSFFloat)node.getEventIn("minBack");
		_set_priority = ( EventInSFFloat)node.getEventIn("priority");
		_set_source = ( EventInSFNode)node.getEventIn("source");

		have_browser = true;
		type = VrmlTypes.Sound;
	}

	public void cleanup()
	{
		if(have_browser)
			_set_source.setValue(null);

		if(_source != null)
			_source.cleanup();

		_source = null;
	}

	public void writeToFile(PrintStream fp, int indent)
	{
		int	i;
		StringBuffer buffer = new StringBuffer();

		for(i = 0; i < indent; i++)
			buffer.append("  ");

		if(name != null)
			fp.print("DEF " + name + " ");

		fp.println(buffer.toString() + "Sound {");

		if((_direction[0] != 0) || (_direction[1] != 0) || (_direction[2] != 1))
			fp.println(buffer.toString() + "  direction " +
					   _direction[0] + " " +
					   _direction[1] + " " +
					   _direction[2]);

		if(_intensity != 1)
			fp.println(buffer.toString() + "  intenstity " + _intensity);

		if((_location[0] != 0) || (_location[1] != 0) || (_location[2] != 1))
			fp.println(buffer.toString() + "  location " +
					   _location[0] + " " +
					   _location[1] + " " +
					   _location[2]);

		if(_maxBack != 10)
			fp.println(buffer.toString() + "  maxBack " + _maxBack);
		if(_maxFront != 10)
			fp.println(buffer.toString() + "  maxFront " + _maxFront);
		if(_minBack != 1)
			fp.println(buffer.toString() + "  minBack " + _minBack);
		if(_minFront != 1)
			fp.println(buffer.toString() + "  minFront " + _minFront);

		if(_priority != 0)
			fp.println(buffer.toString() + "  priority " + _priority);
		if(!_spatialize)
			fp.println(buffer.toString() + "  spatialize FALSE");

		if(_source != null)
			_source.writeToFile(fp, indent + 2);
	}

	public void set_direction(float[] direction)
	{
		_direction = direction;
		if(have_browser)
			_set_direction.setValue(direction);
	}

	public float[] get_direction()
	{
		return(_direction);
	}

	public void set_intensity(float val)
	{
		_intensity = val;
		if(have_browser)
			_set_intensity.setValue(val);
	}

	public float get_intensity()
	{
		return(_intensity);
	}

	public void set_location(float[] location)
	{
		_location = location;
		if(have_browser)
			_set_location.setValue(location);
	}

	public float[] get_location()
	{
		return(_location);
	}

	public void set_maxBack(float val)
	{
		_maxBack = val;
		if(have_browser)
			_set_maxBack.setValue(val);
	}

	public float get_maxBack()
	{
		return(_maxBack);
	}

	public void set_maxFront(float val)
	{
		_maxFront = val;
		if(have_browser)
			_set_maxFront.setValue(val);
	}

	public float get_maxFront()
	{
		return(_maxFront);
	}

	public void set_minBack(float val)
	{
		_minBack = val;
		if(have_browser)
			_set_minBack.setValue(val);
	}

	public float get_minBack()
	{
		return(_minBack);
	}

	public void set_minFront(float val)
	{
		_minFront = val;
		if(have_browser)
			_set_minFront.setValue(val);
	}

	public float get_minFront()
	{
		return(_minFront);
	}

	public void set_priority(float priority)
	{
		_priority = priority;
		if(have_browser)
			_set_priority.setValue(priority);
	}

	public float get_priority()
	{
		return(_priority);
	}

	public void set_source(VrmlObject source)
	{
		if(!(source instanceof AudioNode))
			return;

		_source = source;
		if(have_browser)
			_set_source.setValue(source.node);
	}

	// this should really be returning a AudioNode but..... 
	public VrmlObject get_source()
	{
		return(_source);
	}

	public boolean get_spatialize()
	{
		return(_spatialize);
	}
}
