// VRML Generator
// Copyright Justin Couch 1996
//
// Chapter 13: Late Night VRML 2.0 and Java
//
// Switch class

package vlc.vrml.external.geometry;

import java.io.*;
import java.util.*;
import vrml.external.*;
import vrml.external.field.*;

public class Switch extends SpecialGroup
{
	private EventInMFNode	_set_choice;
	private EventInSFInt32	_set_whichChoice;

	private int	num_choice;
	private int	_whichChoice;
	private Vector	_choice;

	public Switch()
	{
		_choice = new Vector();

		Browser b = Browser.getBrowser();

		if(b == null)
			return;

		node = b.createVrmlFromString("Switch {}")[0];

		createFields();
	}

	public Switch(Browser b)
	{
		_choice = new Vector();

		if(b == null)
			return;

		node = b.createVrmlFromString("Switch {}")[0];

		createFields();
	}

	private void createFields()
	{
		_set_choice = (EventInMFNode)node.getEventIn("url");

		_set_whichChoice = (EventInSFInt32)node.getEventIn("whichChoice");

		have_browser = true;
		type = vlc.vrml.VrmlTypes.Switch;
	}

	public void cleanup()
	{
		// force the clean up of internally allocated objects
		int	i;

		for(i = 0; i < _choice.size(); i++)
			removeChoice((ChildrenNode)_choice.elementAt(i));

		_choice = null;
	}

	public void writeToFile(PrintStream fp, int indent)
	{
		int	i;
		StringBuffer buffer = new StringBuffer();

		for(i = 0; i < indent; i++)
			buffer.append("  ");

		fp.print(buffer.toString());
		if(name != null)
			fp.print("DEF " + name + " ");

		fp.println("Switch {");

		// print the contents of the node

		if(_whichChoice != -1)
			fp.println(buffer.toString() +
					   "  whichChoice " +
			            _whichChoice);
			            
		if(num_choice != 0)
		{
			Enumeration e = _choice.elements();

			fp.print(buffer.toString() + "  children " );
			if(num_choice != 1)
				fp.println("[");

			while(e.hasMoreElements())
				((VrmlObject)(e.nextElement())).writeToFile(fp, indent + 2);

			if(num_choice != 1)
				fp.println(buffer.toString() + "  ]");
		}
	}

	public void addChoice(ChildrenNode child)
	{
		int		i;
		Node[]	node_list;

		_choice.addElement(child);

		if(have_browser)
		{
			node_list = new Node[_choice.size()];

			// copy the values across.
			for(i = 0; i < _choice.size(); i++)
				node_list[i] = ((VrmlObject)_choice.elementAt(i)).node;
			
			_set_choice.setValue(node_list);
		}
		
		num_choice++;
	}

	public void removeChoice(ChildrenNode child)
	{
		int		i;
		Node[] 	node_list;

		child.cleanup();
		
		_choice.removeElement(child);

		num_choice--;
		
		if(have_browser)
		{
			if(num_choice == 0)
			{
				_set_choice.setValue(null);
				return;
			}

			node_list = new Node[_choice.size()];

			// copy the values across.
			for(i = 0; i < _choice.size(); i++)
				node_list[i] = ((VrmlObject)_choice.elementAt(i)).node;
			
			_set_choice.setValue(node_list);
		}
		
	}

	public void set_choice(ChildrenNode[] child_list)
	{
		int	i;
		int num = child_list.length;
		Node[] 	node_list;

		// replace the current _children list with the new list.
		// force the garbage collection
		_choice = null;

		// assign the new arrays.
		_choice = new Vector(num);
		node_list = new Node[num];

		// copy the values across.
		for(i = 0; i < num; i++)
		{
			_choice.addElement(child_list[i]);
			node_list[i] = child_list[i].node;
		}

		// set the values in the VRML scene
		if(have_browser)
			_set_choice.setValue(node_list);
	}
	
	public void set_whichChoice(int val)
	{
		_whichChoice = val;
		if(have_browser)
			_set_whichChoice.setValue(val);
	}
	
	public int get_whichChoice()
	{
		return(_whichChoice);
	}

	public ChildrenNode[] get_Choice()
	{
		int	i;
		int	size = _choice.size();
		ChildrenNode[] node_list = new ChildrenNode[size];

		for(i = 0; i < size; i++)
		{
			node_list[i] = (ChildrenNode)_choice.elementAt(i);
		}

		return(node_list);
	}
}
