// VRML Generator
// Copyright Justin Couch 1996
//
// Chapter 13: Late Night VRML 2.0 and Java
//
// TimeSensor class

package vlc.vrml.external.geometry;

import java.io.*;
import vrml.external.*;
import vrml.external.field.*;

public class TimeSensor extends Sensor
{
	private EventInSFTime	_set_cycleInterval;
	private EventInSFTime	_set_startTime;
	private EventInSFTime	_set_stopTime;
	private EventInSFBool	_set_loop;

	private EventOutSFTime	_get_cycleTime;
	private EventOutSFFloat	_get_fraction_changed;
	private EventOutSFTime	_get_time;

	private double	_cycleInterval = 1;
	private double	_startTime = 0;
	private double	_stopTime = 0;
	private boolean	_loop = false;

	public TimeSensor()
	{
		Browser b = Browser.getBrowser();

		if(b == null)
			return;

		node = b.createVrmlFromString("TimeSensor {}")[0];

		createFields();
	}

	public TimeSensor(Browser b)
	{
		if(b == null)
			return;

		node = b.createVrmlFromString("TimeSensor {}")[0];

		createFields();
	}

	private void createFields()
	{
		_set_enabled = (EventInSFBool)node.getEventIn("enabled");
		_set_cycleInterval = (EventInSFTime)node.getEventIn("cycleInterval");
		_set_startTime = (EventInSFTime)node.getEventIn("startTime");
		_set_stopTime = (EventInSFTime)node.getEventIn("stopTime");
		_set_loop = (EventInSFBool)node.getEventIn("loop");

		_get_isActive = (EventOutSFBool)node.getEventOut("isActive");
		_get_cycleTime = (EventOutSFTime)node.getEventOut("cycleTime");
		_get_fraction_changed = (EventOutSFFloat)node.getEventOut("fraction_changed");
		_get_time = (EventOutSFTime)node.getEventOut("time");

		have_browser = true;
		type = vlc.vrml.VrmlTypes.TimeSensor;
	}

	public void writeToFile(PrintStream fp, int indent)
	{
		int	i;
		StringBuffer buffer = new StringBuffer();

		for(i = 0; i < indent; i++)
			buffer.append("  ");

		if(name != null)
			fp.print("DEF " + name + " ");

		fp.println(buffer.toString() + "TimeSensor {");

		if(!_enabled)
			fp.println(buffer.toString() + "  enabled FALSE");

		if(_cycleInterval != 1)
			fp.println(buffer.toString() +
					   "  cycleInterval " +
					   _cycleInterval);

		if(_startTime != 0)
			fp.println(buffer.toString() +
					   "  startTime " +
					   _startTime);

		if(_stopTime != 0)
			fp.println(buffer.toString() +
					   "  stopTime " +
					   _stopTime);

		if(_loop)
			fp.println(buffer.toString() + "  loop TRUE");

		fp.println(buffer.toString() + "}");
	}

	public void set_cycleInterval(double time)
	{
		_cycleInterval = time;
		if(have_browser)
			_set_cycleInterval.setValue(time);
	}

	public double get_cycleInterval()
	{
		return(_cycleInterval);
	}

	public void set_startTime(double pos)
	{
		_startTime = pos;
		if(have_browser)
			_set_startTime.setValue(pos);
	}

	public double get_startTime()
	{
		return(_startTime);
	}

	public void set_stopTime(double time)
	{
		_stopTime = time;
		if(have_browser)
			_set_stopTime.setValue(time);
	}

	public double get_stopTime()
	{
		return(_stopTime);
	}

	public void set_loop(boolean val)
	{
		_loop = val;
		if(have_browser)
			_set_loop.setValue(val);
	}

	public boolean get_loop()
	{
		return(_loop);
	}

	public double get_cycleTime()
	{
		return(_get_cycleTime.getValue());
	}

	public float get_fraction_changed()
	{
		return(_get_fraction_changed.getValue());
	}

	public double get_time()
	{
		return(_get_time.getValue());
	}
}