// VRML Generator
// Copyright Justin Couch 1996
//
// Chapter 13: Late Night VRML 2.0 and Java
//
// Viewpoint class

package vlc.vrml.external.geometry;

import java.io.*;
import vrml.external.*;
import vrml.external.field.*;

public class Viewpoint extends BindableNode
{
	private EventInSFFloat	_set_fieldOfView;
	private EventInSFBool	_set_jump;
	private EventInSFRotation	_set_orientation;
	private EventInSFVec3f	_set_position;

	private EventOutSFTime	_get_bindTime_changed;

	private float	_fieldOfView = 0.785398f;
	private boolean _jump = true;
	private float[]	_orientation = {0, 0, 1, 0};
	private float[]	_position = {0, 0, 0};
	private String	_description = null;

	public Viewpoint()
	{
		Browser b = Browser.getBrowser();

		if(b == null)
			return;

		node = b.createVrmlFromString("Viewpoint {}")[0];

		createFields();
	}

	public Viewpoint(Browser b)
	{
		if(b == null)
			return;

		node = b.createVrmlFromString("Viewpoint {}")[0];

		createFields();
	}

	public Viewpoint(String description)
	{
		String vrml_string;

		Browser b = Browser.getBrowser();

		if(b == null)
			return;

		_description = description;

		vrml_string = "Viewpoint { ";

		if(description != null)
			vrml_string += "description " + description;

		vrml_string += " }";

		node = b.createVrmlFromString(vrml_string)[0];

		createFields();
	}

	public Viewpoint(Browser b, String description)
	{
		String vrml_string;

		if(b == null)
			return;

		_description = description;

		vrml_string = "Viewpoint { ";

		if(description != null)
			vrml_string += "description " + description;

		vrml_string += " }";

		node = b.createVrmlFromString(vrml_string)[0];

		createFields();
	}

	private void createFields()
	{
		_set_bind = (EventInSFBool)node.getEventIn("set_bind");
		_set_fieldOfView = (EventInSFFloat)node.getEventIn("fieldOfView");
		_set_jump = (EventInSFBool)node.getEventIn("jump");
		_set_orientation = (EventInSFRotation)node.getEventIn("orientation");
		_set_position = (EventInSFVec3f)node.getEventIn("position");

		_get_bindTime_changed = (EventOutSFTime)node.getEventOut("bindTime_changed");
		_get_isBound = (EventOutSFBool)node.getEventOut("isBound");

		have_browser = true;

		type = vlc.vrml.VrmlTypes.Viewpoint;
	}

	public void writeToFile(PrintStream fp, int indent)
	{
		int	i;
		StringBuffer buffer = new StringBuffer();

		for(i = 0; i < indent; i++)
			buffer.append("  ");

		if(name != null)
			fp.print("DEF " + name + " ");

		fp.println(buffer.toString() + "Viewpoint {");

		if(_fieldOfView != 0.785398f)
			fp.println(buffer.toString() + "  fieldOfView "	+ _fieldOfView);

		if(!_jump)
			fp.println(buffer.toString() + "  jump FALSE");

		if((_orientation[0] != 0) || (_orientation[1] != 0) ||
		   (_orientation[2] != 1) || (_orientation[3] != 0))
				fp.println(buffer.toString() + "  orientation " +
						   _orientation[0] + " " +
						   _orientation[1] + " " +
						   _orientation[2] + " " +
						   _orientation[3]);

		if((_position[0] != 0) || (_position[1] != 0) || (_position[2] != 1))
				fp.println(buffer.toString() + "  position " +
						   _position[0] + " " +
						   _position[1] + " " +
						   _position[2]);

		if(_description != null)
			fp.println(buffer.toString() + "  description " + _description);

		fp.println(buffer.toString() + "}");
	}

	public void set_fieldOfView(float fov)
	{
		_fieldOfView = fov;
		if(have_browser)
			_set_fieldOfView.setValue(fov);
	}

	public float get_fieldOfView()
	{
		return(_fieldOfView);
	}

	public void set_jump(boolean on)
	{
		_jump = on;
		if(have_browser)
			_set_jump.setValue(on);
	}

	public boolean get_jump()
	{
		return(_jump);
	}

	public void set_orientation(float[] orientation)
	{
		_orientation = orientation;
		if(have_browser)
			_set_orientation.setValue(orientation);
	}

	public float[] get_orientation()
	{
		return(_orientation);
	}

	public void set_position(float[] position)
	{
		_position = position;
		if(have_browser)
			_set_position.setValue(position);
	}

	public float[] get_position()
	{
		return(_position);
	}

	public double get_bindTime_changed()
	{
		return(_get_bindTime_changed.getValue());
	}
}
