// VRML Generator
// Copyright Justin Couch 1996
//
// Chapter 13: Late Night VRML 2.0 and Java
//
// VisibilitySensor class
// We need to override the _offset and _set_offset methods and variables here
// because in the GeometrySensor they are a 4 value SFRotation rather than the
// required SFVec3f by the VisibilitySensor.

package vlc.vrml.external.geometry;

import java.io.*;
import vrml.external.*;
import vrml.external.field.*;

public class VisibilitySensor extends Sensor
{
	private EventInSFVec3f	_set_center;
	private EventInSFVec3f	_set_size;

	private EventOutSFTime	_get_enterTime;
	private EventOutSFTime	_get_exitTime;

	private float[]	_center = {0, 0, 0};
	private float[]	_size = {0, 0, 0};

	public VisibilitySensor()
	{
		Browser b = Browser.getBrowser();

		if(b == null)
			return;

		node = b.createVrmlFromString("VisibilitySensor {}")[0];

		createFields();
	}

	public VisibilitySensor(Browser b)
	{
		if(b == null)
			return;

		node = b.createVrmlFromString("VisibilitySensor {}")[0];

		createFields();
	}

	private void createFields()
	{
		_set_enabled = (EventInSFBool)node.getEventIn("enabled");
		_set_center = (EventInSFVec3f)node.getEventIn("center");
		_set_size = (EventInSFVec3f)node.getEventIn("size");

		_get_isActive = (EventOutSFBool)node.getEventOut("isActive");
		_get_enterTime = (EventOutSFTime)node.getEventOut("enterTime");
		_get_exitTime = (EventOutSFTime)node.getEventOut("exitTime");

		have_browser = true;
		type = vlc.vrml.VrmlTypes.VisibilitySensor;
	}

	public void writeToFile(PrintStream fp, int indent)
	{
		int	i;
		StringBuffer buffer = new StringBuffer();

		for(i = 0; i < indent; i++)
			buffer.append("  ");

		if(name != null)
			fp.print("DEF " + name + " ");

		fp.println(buffer.toString() + "VisibilitySensor {");

		if(!_enabled)
			fp.println(buffer.toString() + "  enabled FALSE");

		if((_center[0] != 0) || (_center[1] != 0) || (_center[2] != 0))
			fp.println(buffer.toString() + "  center " +
			           _center[0] + " " +
			           _center[1] + " " +
			           _center[2]);
			
		if((_size[0] != 0) || (_size[1] != 0) || (_size[2] != 0))
			fp.println(buffer.toString() + "  size " +
			           _size[0] + " " +
			           _size[1] + " " +
			           _size[2]);

		fp.println(buffer.toString() + "}");
	}
	
	public void set_size(float[] pos)
	{
		_size = pos;
		if(have_browser)
			_set_size.setValue(pos);
	}

	public float[] get_size()
	{
		return(_size);
	}
	
	public void set_center(float[] pos)
	{
		_center = pos;
		if(have_browser)
			_set_center.setValue(pos);
	}

	public float[] get_center()
	{
		return(_center);
	}
	public double get_enterTime()
	{
		return(_get_enterTime.getValue());
	}

	public double get_exitTime()
	{
		return(_get_exitTime.getValue());
	}

}