// VRML Generator
// Copyright Justin Couch 1996
//
// Chapter 13: Late Night VRML 2.0 and Java
//
// Cylinder class

package vlc.vrml.internal.geometry;

import java.io.*;
import vrml.*;
import vrml.field.*;
import vrml.node.Node;

public class Cylinder extends Geometry
{
	private boolean	isDefault = true;

	private float _radius = 1;
	private float _height = 2;
	private boolean _side = true;
	private boolean _top = true;
	private boolean _bottom = true;

	public Cylinder(Browser b)
	{
		if(b == null)
			return;

		try
		{
			node = (Node)b.createVrmlFromString("Cylinder {}")[0];
		}
		catch(InvalidVRMLSyntaxException e)
		{
		}
	}

	public Cylinder(Browser b,
					float radius,
					float height,
					boolean showSide,
					boolean showTop,
					boolean showBottom)
	{
		_radius = radius;
		_height = height;
		_side = showSide;
		_top = showTop;
		_bottom = showBottom;

		isDefault = false;

		String vrml_string = "Cylinder {";

		if(radius != 1)
			vrml_string += " radius " + radius;

		if(height != 2)
			vrml_string += " height " + height;

		if(!showSide)
			vrml_string += " side FALSE";

		if(!showTop)
			vrml_string += " top FALSE";

		if(!showBottom)
			vrml_string += "bottom FALSE";

		vrml_string += " }";

		if(b == null)
			return;

		try
		{
			node = (Node)b.createVrmlFromString(vrml_string)[0];
		}
		catch(InvalidVRMLSyntaxException e)
		{
		}

		have_browser = true;

	}

	public void writeToFile(PrintStream fp, int indent)
	{
		int	i;
		StringBuffer buffer = new StringBuffer();

		for(i = 0; i < indent; i++)
			buffer.append("  ");

		fp.print(buffer.toString() + "geometry ");
		if(name != null)
			fp.print("DEF " + name + " ");


		if(isDefault)
		{
			fp.println("Cylinder {}");
			return;
		}

		fp.println("Cylinder {");

		if(_radius != 1)
			fp.println(buffer.toString() +
					   " bottomRadius " +
					   _radius);

		if(_height != 2)
			fp.println(buffer.toString() +
					   " height " +
					   _height);
		if(!_side)
			fp.println(buffer.toString() + " side FALSE");

		if(!_top)
			fp.println(buffer.toString() + " top FALSE");

		if(!_bottom)
			fp.println(buffer.toString() + " bottom FALSE ");

		fp.println(buffer.toString() + "}");
	}

	public float get_radius()
	{
		return(_radius);
	}

	public float get_height()
	{
		return(_height);
	}

	public boolean get_side()
	{
		return(_side);
	}

	public boolean get_top()
	{
		return(_top);
	}

	public boolean get_bottom()
	{
		return(_bottom);
	}


}
