// VRML Generator
// Copyright Justin Couch 1996
//
// Chapter 13: Late Night VRML 2.0 and Java
//
// Fog class

package vlc.vrml.internal.geometry;

import java.io.*;
import vrml.*;
import vrml.field.*;
import vrml.node.Node;

public class Fog extends BindableNode
{
	private SFColor	_set_color;
	private SFString	_set_fogType;
	private SFFloat	_set_visibilityRange;

	private float[]	_color = { 1, 1, 1};
	private String	_fogType = "LINEAR";
	private float	_visibilityRange = 1000;

	public Fog(Browser b)
	{
		if(b == null)
			return;

		try
		{
			node = (Node)b.createVrmlFromString("Fog {}")[0];
		}
		catch(InvalidVRMLSyntaxException e)
		{
		}

		createFields();
	}

	private void createFields()
	{
		_set_bind = (SFBool)node.getEventIn("set_bind");
		_set_fogType = (SFString)node.getEventIn("fogType");
		_set_color = (SFColor)node.getEventIn("color");
		_set_visibilityRange = (SFFloat)node.getEventIn("visibilityRange");

		_get_isBound = (ConstSFBool)node.getEventOut("isBound");

		have_browser = true;
		type = vlc.vrml.VrmlTypes.Fog;
	}

	public void writeToFile(PrintStream fp, int indent)
	{
		int	i;
		StringBuffer buffer = new StringBuffer();

		for(i = 0; i < indent; i++)
			buffer.append("  ");

		if(name != null)
			fp.print("DEF " + name + " ");

		fp.println(buffer.toString() + "Fog {");

		if((_color[0] != 1) || (_color[1] != 1) || (_color[2] != 1))
			fp.println(buffer.toString() + "  color " +
					   _color[0] + " " +
					   _color[1] + " " +
					   _color[2]);

		if(!_fogType.equals("LINEAR"))
			fp.println(buffer.toString() +  "  fogType " + _fogType);

		if(_visibilityRange != 1000)
			fp.println(buffer.toString() + "  visibilityRange " + _visibilityRange);

		fp.println(buffer.toString() + "}");
	}

	public void set_color(float red, float green, float blue)
	{
		float[] c = new float[3];

		c[0] = red;
		c[1] = green;
		c[2] = blue;

		_color = c;
		if(have_browser)
			_set_color.setValue(c);
	}

	public float[] get_color()
	{
		return(_color);
	}

	public void set_fogType(String type)
	{
		_fogType = type;
		if(have_browser)
			_set_fogType.setValue(type);
	}

	public String get_fogType()
	{
		return(_fogType);
	}

	public void set_visiblityRange(float range)
	{
		_visibilityRange = range;
		if(have_browser)
			_set_visibilityRange.setValue(range);
	}

	public float get_VisibilityRange()
	{
		return(_visibilityRange);
	}
}