// VRML Generator
// Copyright Justin Couch 1996
//
// Chapter 25: Late Night VRML 2.0 and Java
//
// FontStyle class.

package vlc.vrml.internal.geometry;

import java.io.PrintStream;
import vrml.*;
import vrml.field.*;
import vrml.node.Node;

public class FontStyle extends VrmlObject
{
	// no need to set initial values because these will always be set
	// in the constructor (the are all field type access
	private String	_family;
	private boolean _horizontal;
	private String	_justify;
	private String  _language;
	private boolean _leftToRight;
	private float   _size;
	private float   _spacing;
	private String  _style;
	private boolean	_topToBottom;


	// this is the one case where the default constuctors are useless.
	// so don't put them in!

	public FontStyle(Browser b,
					 String	 family,
					 boolean horizontal,
					 String	 justify,
					 String  language,
					 boolean leftToRight,
					 float   size,
					 float   spacing,
					 String  style,
					 boolean topToBottom)
	{
		_family = family;
		_horizontal = horizontal;
		_justify = justify;
		_language = language;
		_leftToRight = leftToRight;
		_size = size;
		_spacing = spacing;
		_style = style;
		_topToBottom = topToBottom;

		if(b == null)
			return;

		try
		{
			node = (Node)b.createVrmlFromString(createString())[0];
		}
		catch(InvalidVRMLSyntaxException e)
		{
		}

		have_browser = true;
		type = vlc.vrml.VrmlTypes.FontStyle;
	}

	private String createString()
	{
		int	i;
		String vrml_string = "FontStyle { ";

		if(!_family.equals("SERIF"))
			vrml_string += " family " + _family;

		if(!_horizontal)
			vrml_string += " horizontal FALSE";

		if(!_justify.equals("BEGIN"))
			vrml_string += " justify " + _justify;

		if(_language != null)
			vrml_string += " language " + _language;

		if(!_leftToRight)
			vrml_string += " leftToRight FALSE";

		if(_size != 1)
			vrml_string += " size " + _size;

		if(_spacing != 1)
			vrml_string += " spacing " + _spacing;

		if(_style != null)
			vrml_string += " style " + _style;

		if(!_topToBottom)
			vrml_string += " topToBottom FALSE";

		vrml_string += "}";

		return(vrml_string);
	}

	public void cleanup()
	{
		_family = null;
		_justify = null;
		_language = null;
		_style = null;
	}

	public void writeToFile(PrintStream fp, int indent)
	{
		int     i;
		StringBuffer buffer = new StringBuffer();

		for(i = 0; i < indent; i++)
			buffer.append("  ");

		fp.print(buffer.toString() + "texture ");
		if(name != null)
			fp.print("DEF " + name + " ");

		fp.println("FontStyle {");

		if(!_family.equals("SERIF"))
			fp.println(buffer.toString() + "  family " + _family);

		if(!_horizontal)
			fp.println(buffer.toString() + "  horizontal FALSE");

		if(!_justify.equals("BEGIN"))
			fp.println(buffer.toString() + "  justify " + _justify);

		if(_language != null)
			fp.println(buffer.toString() + "  language " + _language);

		if(!_leftToRight)
			fp.println(buffer.toString() + "  leftToRight FALSE");

		if(_size != 1)
			fp.println(buffer.toString() + "  size " + _size);

		if(_spacing != 1)
			fp.println(buffer.toString() + "  spacing " + _spacing);

		if(_style != null)
			fp.println(buffer.toString() + "  style " + _style);

		if(!_topToBottom)
			fp.println(buffer.toString() + "  topToBottom FALSE");

		fp.println(buffer.toString() + "}");
	}

	public String get_family()
	{
		return(_family);
	}

	public boolean get_horizontal()
	{
		return(_horizontal);
	}

	public String get_justify()
	{
		return(_justify);
	}

	public String get_language()
	{
		return(_language);
	}

	public boolean get_leftToRight()
	{
		return(_leftToRight);
	}

	public float get_size()
	{
		return(_size);
	}

	public float get_spacing()
	{
		return(_spacing);
	}

	public String get_style()
	{
		return(_style);
	}

	public boolean get_topToBottom()
	{
		return(_topToBottom);
	}
}
