// VRML Generator
// Copyright Justin Couch 1996
//
// Chapter 13: Late Night VRML 2.0 and Java
//
// Material class

package vlc.vrml.internal.geometry;

import java.io.*;
import vrml.*;
import vrml.field.*;
import vrml.node.Node;

public class Material extends VrmlObject
{
	private SFColor	_set_diffuse;
	private SFColor	_set_emissive;
	private SFColor	_set_specular;
	private SFFloat	_set_ambient;
	private SFFloat	_set_shine;
	private SFFloat	_set_transparent;

	private float[]	diffuse = {0.8f, 0.8f, 0.8f};
	private float[]	emissive = {0, 0, 0};
	private float[]	specular = {0, 0, 0};
	private float	ambient = 0.2f;
	private float	shininess = 0.2f;
	private float	transparency = 0;
	private boolean diffuse_changed = false;
	private boolean emissive_changed = false;
	private boolean specular_changed = false;

	public Material(Browser b)
	{
		if(b == null)
			return;

		try
		{
			node = (Node)b.createVrmlFromString("Material {}")[0];
		}
		catch(InvalidVRMLSyntaxException e)
		{
		}

		createFields();
	}

	private void createFields()
	{
		_set_ambient = (SFFloat)node.getEventIn("ambientIntensity");
		_set_diffuse = (SFColor)node.getEventIn("diffuseColor");
		_set_emissive = (SFColor)node.getEventIn("emissiveColor");
		_set_specular = (SFColor)node.getEventIn("specularColor");
		_set_shine = (SFFloat)node.getEventIn("shininess");
		_set_transparent = (SFFloat)node.getEventIn("transparency");

		have_browser = true;
		type = vlc.vrml.VrmlTypes.Material;
	}

	public void writeToFile(PrintStream fp, int indent)
	{
		int	i;
		StringBuffer buffer = new StringBuffer();

		for(i = 0; i < indent; i++)
			buffer.append("  ");

		fp.print(buffer.toString() + "material ");
		if(name != null)
			fp.print("DEF " + name + " ");

		fp.println("Material {");

		// print the contents of the shape node
		if(ambient != 0.2f)
			fp.println(buffer.toString() + " ambient " + ambient);

		if(shininess != 0.2f)
			fp.println(buffer.toString() +
				   " shininess " + shininess);

		if(transparency != 0)
			fp.println(buffer.toString() +
				   " transparency " + ambient);

		if(diffuse_changed)
			fp.println(buffer.toString() +
				   " diffuseColor " +
				   diffuse[0] + " " +
				   diffuse[1] + " " +
				   diffuse[2]);

		if(emissive_changed)
			fp.println(buffer.toString() +
				   " emissiveColor " +
				   emissive[0] + " " +
				   emissive[1] + " " +
				   emissive[2]);

		if(specular_changed)
			fp.println(buffer.toString() +
				   " specularColor " +
				   specular[0] + " " +
				   specular[1] + " " +
				   specular[2]);

		fp.println(buffer.toString() + "}");
	}

	public void set_ambientIntensity(float val)
	{
		ambient = val;
		if(have_browser)
			_set_ambient.setValue(val);
	}

	public float get_ambientIntensity()
	{
		return(ambient);
	}

	public void set_shininess(float val)
	{
		shininess = val;
		if(have_browser)
			_set_shine.setValue(val);
	}

	public float get_shininess()
	{
		return(shininess);
	}

	public void set_transparency(float val)
	{
		transparency = val;
		if(have_browser)
			_set_transparent.setValue(val);
	}

	public float get_transparency()
	{
		return(transparency);
	}

	public void set_diffuseColor(float red, float green, float blue)
	{
		diffuse[0] = red;
		diffuse[1] = green;
		diffuse[2] = blue;

		if(have_browser)
			_set_diffuse.setValue(diffuse);

		if((red == 0.8) && (green == 0.8) && (blue == 0.8))
			diffuse_changed = false;
		else
			diffuse_changed = true;
	}

	public float[] get_diffuseColor()
	{
		return(diffuse);
	}

	public void set_emissiveColor(float red, float green, float blue)
	{
		emissive[0] = red;
		emissive[1] = green;
		emissive[2] = blue;

		if(have_browser)
			_set_emissive.setValue(emissive);

		if((red == 0) && (green == 0) && (blue != 0))
			emissive_changed = false;
		else
			emissive_changed = true;
	}

	public float[] get_emissiveColor()
	{
		return(emissive);
	}

	public void set_specularColor(float red, float green, float blue)
	{
		specular[0] = red;
		specular[1] = green;
		specular[2] = blue;

		if(have_browser)
			_set_specular.setValue(specular);

		if((red == 0) && (green == 0) && (blue == 0))
			specular_changed = false;
		else
			specular_changed = true;
	}

	public float[] get_specularColor()
	{
		return(specular);
	}

}
