// VRML Generator
// Copyright Justin Couch 1996
//
// Chapter 25: Late Night VRML 2.0 and Java
//
// PixelTexture class

package vlc.vrml.internal.geometry;

import java.io.PrintStream;
import vrml.*;
import vrml.field.*;
import vrml.node.Node;

public class PixelTexture extends Texture
{
	private SFImage	_set_image;

	private int		_width;
	private int		_height;
	private int		_num_components;
	private int[]	_pixels;

	public PixelTexture(Browser b)
	{
		if(b == null)
			return;

		try
		{
			node = (Node)b.createVrmlFromString("PixelTexture {}")[0];
		}
		catch(InvalidVRMLSyntaxException e)
		{
		}

		createFields();
	}

	public PixelTexture(Browser b, boolean rptS, boolean rptT)
	{
		String  tmp = "PixelTexture { ";

		repeatS = rptS;
		repeatT = rptT;

		if(!repeatS)
			tmp += "repeatS FALSE ";
		if(!repeatT)
			tmp += "repeatT FALSE ";

		tmp += "}";

		if(b == null)
			return;

		try
		{
			node = (Node)b.createVrmlFromString(tmp)[0];
		}
		catch(InvalidVRMLSyntaxException e)
		{
		}

		createFields();
	}

	private void createFields()
	{
		_set_image = (SFImage)node.getEventIn("image");

		have_browser = true;
		type = vlc.vrml.VrmlTypes.PixelTexture;
	}

	public void cleanup()
	{
		_pixels = null;

	}

	public void writeToFile(PrintStream fp, int indent)
	{
		int     i;
		StringBuffer buffer = new StringBuffer();

		for(i = 0; i < indent; i++)
			buffer.append("  ");

		fp.print(buffer.toString() + " texture ");
		if(name != null)
			fp.print("DEF " + name + " ");

		fp.println("PixelTexture {");

		if(!repeatS)
			fp.println(buffer.toString() + " repeatS FALSE");
		if(!repeatT)
			fp.println(buffer.toString() + " repeatT FALSE");

		if((_width != 0) && (_height != 0))
		{
			fp.println(buffer.toString() + " image " + _width +
						" " +  _height + " " + _num_components);

			int num_vals = _width * _height;

			fp.print(buffer.toString() + "    ");
			for(i = 0; i < num_vals; i++)
			{
				fp.print(" " + Integer.toHexString(_pixels[i]));
				if((i % 4) == 0)
				{
					fp.println("");
					fp.print(buffer.toString() + "    ");
				}
			}
		}
	}

	public void set_image(int width,
						  int height,
						  int num_components,
						  int[] pixels)
	{
		_width = width;
		_height = height;
		_num_components = num_components;

		// now will this work? Should I make an internal copy or not?
		_pixels = pixels;

		// this is causing problems - because CosmoPC has implemented this
		// setValue as int, int, int, byte[]  which is incorrect according to
		// external interface reference.

//		if(have_browser)
//			_set_image.setValue(width, pixels);
	}

	public int get_width()
	{
		return(_width);
	}

	public int get_height()
	{
		return(_height);
	}

	public int get_numComponents()
	{
		return(_num_components);
	}

	public int[] get_pixels()
	{
		return(_pixels);
	}
}
