// VRML Generator
// (c) Justin Couch 1996
//
// From Chapter 13 Late Night VRML 2.0 and Java
//
// Sound Class.

package vlc.vrml.internal.geometry;

import java.io.PrintStream;
import java.util.*;
import vrml.*;
import vrml.field.*;
import vrml.node.Node;

public class Sound extends ChildrenNode
{
	private SFVec3f	_set_direction;
	private SFFloat	_set_intensity;
	private SFVec3f	_set_location;
	private SFFloat	_set_maxBack;
	private SFFloat	_set_maxFront;
	private SFFloat	_set_minBack;
	private SFFloat	_set_minFront;
	private SFFloat	_set_priority;
	private SFNode	_set_source;

	private float[]	_direction = { 0, 0, 1};
	private float	_intensity = 1;
	private float[]	_location = { 0, 0, 0};
	private float	_maxBack = 10;
	private float	_maxFront = 10;
	private float	_minBack = 1;
	private float	_minFront = 1;
	private float	_priority = 0;
	private VrmlObject	_source = null;	// an instance of AudioNode
	private boolean	_spatialize = true;

	public Sound(Browser b)
	{
		if(b == null)
			return;

		try
		{
			node = (Node)b.createVrmlFromString("Sound {}")[0];
		}
		catch(InvalidVRMLSyntaxException e)
		{
		}

		createFields();
	}

	public Sound(Browser b, boolean spatialize)
	{
		String vrml_string;

		_spatialize = spatialize;

		if(b == null)
			return;

		if(!spatialize)
			vrml_string = "Sound { spatialize FALSE }";
		else
			vrml_string = "Sound";

		try
		{
			node = (Node)b.createVrmlFromString(vrml_string)[0];
		}
		catch(InvalidVRMLSyntaxException e)
		{
		}

		createFields();
	}

	private void createFields()
	{
		_set_direction = (SFVec3f)node.getEventIn("direction");
		_set_intensity = (SFFloat)node.getEventIn("intensity");
		_set_location = (SFVec3f)node.getEventIn("location");
		_set_maxBack = (SFFloat)node.getEventIn("maxBack");
		_set_maxFront = (SFFloat)node.getEventIn("maxFront");
		_set_minBack = (SFFloat)node.getEventIn("minBack");
		_set_minFront = (SFFloat)node.getEventIn("minBack");
		_set_priority = (SFFloat)node.getEventIn("priority");
		_set_source = (SFNode)node.getEventIn("source");

		have_browser = true;
		type = vlc.vrml.VrmlTypes.Sound;
	}

	public void cleanup()
	{
		if(have_browser)
			_set_source.setValue((SFNode)null);

		if(_source != null)
			_source.cleanup();

		_source = null;
	}

	public void writeToFile(PrintStream fp, int indent)
	{
		int	i;
		StringBuffer buffer = new StringBuffer();

		for(i = 0; i < indent; i++)
			buffer.append("  ");

		if(name != null)
			fp.print("DEF " + name + " ");

		fp.println(buffer.toString() + "Sound {");

		if((_direction[0] != 0) || (_direction[1] != 0) || (_direction[2] != 1))
			fp.println(buffer.toString() + "  direction " +
					   _direction[0] + " " +
					   _direction[1] + " " +
					   _direction[2]);

		if(_intensity != 1)
			fp.println(buffer.toString() + "  intenstity " + _intensity);

		if((_location[0] != 0) || (_location[1] != 0) || (_location[2] != 1))
			fp.println(buffer.toString() + "  location " +
					   _location[0] + " " +
					   _location[1] + " " +
					   _location[2]);

		if(_maxBack != 10)
			fp.println(buffer.toString() + "  maxBack " + _maxBack);
		if(_maxFront != 10)
			fp.println(buffer.toString() + "  maxFront " + _maxFront);
		if(_minBack != 1)
			fp.println(buffer.toString() + "  minBack " + _minBack);
		if(_minFront != 1)
			fp.println(buffer.toString() + "  minFront " + _minFront);

		if(_priority != 0)
			fp.println(buffer.toString() + "  priority " + _priority);
		if(!_spatialize)
			fp.println(buffer.toString() + "  spatialize FALSE");

		if(_source != null)
			_source.writeToFile(fp, indent + 2);
	}

	public void set_direction(float[] direction)
	{
		_direction = direction;
		if(have_browser)
			_set_direction.setValue(direction);
	}

	public float[] get_direction()
	{
		return(_direction);
	}

	public void set_intensity(float val)
	{
		_intensity = val;
		if(have_browser)
			_set_intensity.setValue(val);
	}

	public float get_intensity()
	{
		return(_intensity);
	}

	public void set_location(float[] location)
	{
		_location = location;
		if(have_browser)
			_set_location.setValue(location);
	}

	public float[] get_location()
	{
		return(_location);
	}

	public void set_maxBack(float val)
	{
		_maxBack = val;
		if(have_browser)
			_set_maxBack.setValue(val);
	}

	public float get_maxBack()
	{
		return(_maxBack);
	}

	public void set_maxFront(float val)
	{
		_maxFront = val;
		if(have_browser)
			_set_maxFront.setValue(val);
	}

	public float get_maxFront()
	{
		return(_maxFront);
	}

	public void set_minBack(float val)
	{
		_minBack = val;
		if(have_browser)
			_set_minBack.setValue(val);
	}

	public float get_minBack()
	{
		return(_minBack);
	}

	public void set_minFront(float val)
	{
		_minFront = val;
		if(have_browser)
			_set_minFront.setValue(val);
	}

	public float get_minFront()
	{
		return(_minFront);
	}

	public void set_priority(float priority)
	{
		_priority = priority;
		if(have_browser)
			_set_priority.setValue(priority);
	}

	public float get_priority()
	{
		return(_priority);
	}

	public void set_source(VrmlObject source)
	{
		if(!(source instanceof AudioNode))
			return;

		_source = source;
		if(have_browser)
		{
			if(source != null)
				_set_source.setValue(source.node);
			else
				_set_source.setValue((SFNode)null);
		}

	}

	// this should really be returning a AudioNode but..... 
	public VrmlObject get_source()
	{
		return(_source);
	}

	public boolean get_spatialize()
	{
		return(_spatialize);
	}
}
