// VRML Generator
// Copyright Justin Couch 1996
//
// Chapter 13: Late Night VRML 2.0 and Java
//
// SphereSensor class

package vlc.vrml.internal.geometry;

import java.io.*;
import vrml.*;
import vrml.field.*;
import vrml.node.Node;

public class SphereSensor extends GeometrySensor
{
	private SFVec2f	_set_maxPosition;
	private SFVec2f	_set_minPosition;
	private SFRotation	_set_offset;

	private float[]	_maxPosition = {-1, -1};
	private float[]	_minPosition = {0, 0};
	private float[]	_offset = { 0, 0, 0};

	public SphereSensor(Browser b)
	{
		if(b == null)
			return;

		try
		{
			node = (Node)b.createVrmlFromString("SphereSensor {}")[0];
		}
		catch(InvalidVRMLSyntaxException e)
		{
		}

		createFields();
	}

	private void createFields()
	{
		_set_enabled = (SFBool)node.getEventIn("enabled");
		_set_offset = (SFRotation)node.getEventIn("offset");
		_set_autoOffset = (SFBool)node.getEventIn("autoOffset");

		_get_isActive = (ConstSFBool)node.getEventOut("isActive");
		_get_rotation_changed = (ConstSFRotation)node.getEventOut("rotation_changed");
		_get_trackPoint_changed = (ConstSFVec3f)node.getEventOut("trackPoint_changed");

		have_browser = true;
		type = vlc.vrml.VrmlTypes.SphereSensor;
	}

	public void writeToFile(PrintStream fp, int indent)
	{
		int	i;
		StringBuffer buffer = new StringBuffer();

		for(i = 0; i < indent; i++)
			buffer.append("  ");

		if(name != null)
			fp.print("DEF " + name + " ");

		fp.println(buffer.toString() + "SphereSensor {");

		if(!_enabled)
			fp.println(buffer.toString() + "  enabled FALSE");

		if((_offset[0] != 0) || (_offset[1] != 1) || (_offset[2] != 0))
		   	fp.println(buffer.toString() + "  offset " +
		   	           _offset[0] + " " +
		   	           _offset[1] + " " +
		   	           _offset[2]);
		   	           
		if(!_autoOffset)
			fp.println(buffer.toString() + "  autoOffset FALSE");

		fp.println(buffer.toString() + "}");
	}
}