// VRML Generator
// Copyright Justin Couch 1996
//
// Chapter 25: Late Night VRML 2.0 and Java
//
// SpotLight class

package vlc.vrml.internal.geometry;

import java.io.*;
import vrml.*;
import vrml.field.*;
import vrml.node.Node;

public class SpotLight extends Light
{
	private SFVec3f	_set_attenuation;
	private SFVec3f	_set_location;
	private SFFloat	_set_radius;
	private SFVec3f	_set_direction;
	private SFFloat	_set_beamWidth;
	private SFFloat	_set_cutOffAngle;

	private float[]	_attenuation = { 1, 0, 0};
	private float[]	_location = { 0, 0, 0};
	private float	_radius = 100;
	private float[]	_direction = { 0, 0, -1};
	private float	_beamWidth = 1.570796f;
	private float	_cutOffAngle = 0.785398f;

	public SpotLight(Browser b)
	{
		if(b == null)
			return;

		try
		{
			node = (Node)b.createVrmlFromString("SpotLight {}")[0];
		}
		catch(InvalidVRMLSyntaxException e)
		{
		}

		createFields();
	}

	private void createFields()
	{
		_set_on = (SFBool)node.getEventIn("on");
		_set_intensity = (SFFloat)node.getEventIn("intensity");
		_set_ambientIntensity = (SFFloat)node.getEventIn("ambientIntensity");
		_set_color = (SFColor)node.getEventIn("color");
		_set_attenuation = (SFVec3f)node.getEventIn("attenuation");
		_set_location = (SFVec3f)node.getEventIn("location");
		_set_radius = (SFFloat)node.getEventIn("radius");
		_set_direction = (SFVec3f)node.getEventIn("direction");
		_set_beamWidth = (SFFloat)node.getEventIn("beamWidth");
		_set_cutOffAngle = (SFFloat)node.getEventIn("cutOffAngle");

		have_browser = true;
		type = vlc.vrml.VrmlTypes.SpotLight;
	}

	public void writeToFile(PrintStream fp, int indent)
	{
		int	i;
		StringBuffer buffer = new StringBuffer();

		for(i = 0; i < indent; i++)
			buffer.append("  ");

		if(name != null)
			fp.print("DEF " + name + " ");

		fp.println(buffer.toString() + "SpotLight {");

		if(!_on)
			fp.println(buffer.toString() + "  on FALSE");

		if(_ambientIntensity != 0)
			fp.println(buffer.toString() +
					   "  ambientIntensity " +
					   _ambientIntensity);

		if(_intensity != 0)
			fp.println(buffer.toString() +
					   "  intensity " +
					   _intensity);

		if((_color[0] != 1) || (_color[1] != 1) || (_color[2] != 1))
			fp.println(buffer.toString() + "  color "
					   + _color[0] + " " +
					   + _color[1] + " " +
					   + _color[2]);

		if((_attenuation[0] != 1) || (_attenuation[1] != 0) ||
		   (_attenuation[2] != 0))
			fp.println(buffer.toString() + "  attenuation "
					   + _attenuation[0] + " " +
					   + _attenuation[1] + " " +
					   + _attenuation[2]);

		if((_location[0] != 1) || (_location[1] != 0) ||
		   (_location[2] != 0))
			fp.println(buffer.toString() + "  location "
					   + _location[0] + " " +
					   + _location[1] + " " +
					   + _location[2]);

		if((_direction[0] != 0) || (_direction[1] != 0) ||
		   (_direction[2] != -1))
			fp.println(buffer.toString() + "  direction "
					   + _direction[0] + " " +
					   + _direction[1] + " " +
					   + _direction[2]);

		if(_beamWidth != 1.570796f)
			fp.println(buffer.toString() +
					   "  beamWidth " +
					   _beamWidth);

		if(_cutOffAngle != 0.785398f)
			fp.println(buffer.toString() +
					   "  cutOffAngle " +
					   _cutOffAngle);

		if(_radius != 100)
			fp.println(buffer.toString() + "  radius " + _radius);

		fp.println(buffer.toString() + "}");
	}

	public void set_attenuation(float[] val)
	{
		_attenuation = val;
		if(have_browser)
			_set_attenuation.setValue(val);
	}

	public float[] get_attenuation()
	{
		return(_attenuation);
	}

	public void set_location(float[] val)
	{
		_location = val;
		if(have_browser)
			_set_location.setValue(val);
	}

	public float[] get_location()
	{
		return(_location);
	}

	public void set_radius(float val)
	{
		_radius = val;
		if(have_browser)
			_set_radius.setValue(val);
	}

	public float get_radius()
	{
		return(_radius);
	}

	public void set_beamWidth(float val)
	{
		_beamWidth = val;
		if(have_browser)
			_set_beamWidth.setValue(val);
	}

	public float get_beamWidth()
	{
		return(_beamWidth);
	}

	public void set_cutOffAngle(float val)
	{
		_cutOffAngle = val;
		if(have_browser)
			_set_cutOffAngle.setValue(val);
	}

	public float get_cutOffAngle()
	{
		return(_cutOffAngle);
	}

	public void set_direction(float[] val)
	{
		_direction = val;
		if(have_browser)
			_set_direction.setValue(val);
	}

	public float[] get_direction()
	{
		return(_direction);
	}

}