// VRML Generator
// Copyright Justin Couch 1996
//
// Chapter 13: Late Night VRML 2.0 and Java
//
// Transform class
// The get_* methods are very trusting of the user. They pass back pointers to
// the internal data. To be more secure it should clone the array.

package vlc.vrml.internal.geometry;

import java.io.*;
import java.util.*;
import vrml.*;
import vrml.field.*;
import vrml.node.Node;

public class Transform extends GroupingNode
{
	private SFVec3f		_set_center;
	private SFRotation	_set_rotation;
	private SFVec3f		_set_scale;
	private SFRotation	_set_scaleOrientation;
	private SFVec3f		_set_translation;

	private float[]		_center = {0, 0, 0};
	private float[]		_rotation = {0, 0, 1, 0};
	private float[]		_scale = {0, 0, 0};
	private float[] 	_scaleOrientation = {0, 0, 1, 0};
	private float[]		_translation = {0, 0, 0};

	public Transform(Browser b)
	{
		if(b == null)
			return;

		try
		{
			node = (Node)b.createVrmlFromString("Transform {}")[0];
		}
		catch(InvalidVRMLSyntaxException e)
		{
		}

		createFields();
	}

	public Transform(Browser b, float[] bboxCenter, float[] bboxSize)
	{
		if((bboxCenter != null) && (bboxCenter.length > 2))
			_bboxCenter = bboxCenter;

		if((bboxSize != null) && (bboxSize.length > 2))
			_bboxSize = bboxSize;

		if(b == null)
			return;

		try
		{
			node = (Node)b.createVrmlFromString(createString())[0];
		}
		catch(InvalidVRMLSyntaxException e)
		{
		}

		createFields();
	}

	private void createFields()
	{
		_addChildren = (MFNode)node.getEventIn("addChildren");
		_removeChildren = (MFNode)node.getEventIn("removeChildren");
		_set_center = (SFVec3f)node.getEventIn("center");
		_set_children = (MFNode)node.getEventIn("children");
		_set_rotation = (SFRotation)node.getEventIn("rotation");
		_set_scale = (SFVec3f)node.getEventIn("scale");
		_set_scaleOrientation = (SFRotation)node.getEventIn("scaleOrientation");
		_set_translation = (SFVec3f)node.getEventIn("translation");

		have_browser = true;
		type = vlc.vrml.VrmlTypes.Transform;
	}

	private String createString()
	{
		String vrml_string = "Transform { ";

		if((_bboxCenter[0] != 0) || (_bboxCenter[1] != 0) ||
		   (_bboxCenter[2] != 0))
		{
			vrml_string += "bboxCenter " +
							_bboxCenter[0] + " " +
							_bboxCenter[1] + " " +
							_bboxCenter[2] + " ";
		}

		if((_bboxSize[0] != -1) || (_bboxSize[1] != -1) ||
		   (_bboxSize[2] != -1))
		{
			vrml_string += "bboxCenter " +
							_bboxSize[0] + " " +
							_bboxSize[1] + " " +
							_bboxSize[2] + " ";
		}

		vrml_string += " }";

		return(vrml_string);
	}

	public void cleanup()
	{
		int	i;

		// force the clean up of internally allocated objects
		for(i = 0; i < _children.size(); i++)
			removeChildren((ChildrenNode)_children.elementAt(i));

		_center = null;
		_children = null;
		_rotation = null;
		_scale = null;
		_scaleOrientation = null;
		_translation = null;
		_bboxCenter = null;
		_bboxSize = null;
	}

	public void writeToFile(PrintStream fp, int indent)
	{
		int	i;
		StringBuffer buffer = new StringBuffer();

		for(i = 0; i < indent; i++)
			buffer.append("  ");

		fp.print(buffer.toString());
		if(name != null)
			fp.print("DEF " + name + " ");

		fp.println("Transform {");

		// print the contents of the Transform node
		if((_center[0] != 0) || (_center[1] != 0) || (_center[2] != 0))
			fp.println(buffer.toString() + " center " +
						_center[0] + " " +
						_center[1] + " " +
						_center[2]);


		if(num_children != 0)
		{
			Enumeration e = _children.elements();

			fp.print(buffer.toString() + "  children " );
			if(num_children != 1)
				fp.println("[");

			for(;e.hasMoreElements();)
				((VrmlObject)(e.nextElement())).writeToFile(fp, indent + 2);

			if(num_children != 1)
				fp.println(buffer.toString() + "  ]");
		}

		if((_rotation[0] != 0) || (_rotation[1] != 0) ||
		   (_rotation[2] != 1) || (_rotation[3] != 0))
			fp.println(buffer.toString() + " rotation " +
						_rotation[0] + " " +
						_rotation[1] + " " +
						_rotation[2] + " " +
						_rotation[3]);

		if((_scale[0] != 0) || (_scale[1] != 0) || (_scale[2] != 0))
			fp.println(buffer.toString() + " scale " +
						_scale[0] + " " +
						_scale[1] + " " +
						_scale[2]);

		if((_scaleOrientation[0] != 0) || (_scaleOrientation[1] != 0) ||
		   (_scaleOrientation[2] != 1) || (_scaleOrientation[3] != 0))
			fp.println(buffer.toString() + " scaleOrientation " +
						_scaleOrientation[0] + " " +
						_scaleOrientation[1] + " " +
						_scaleOrientation[2] + " " +
						_scaleOrientation[3]);

		if((_translation[0] != 0) || (_translation[1] != 0) ||
		   (_translation[2] != 0))
			fp.println(buffer.toString() + " translation " +
						_translation[0] + " " +
						_translation[1] + " " +
						_translation[2]);

		if((_bboxCenter[0] != 0) || (_bboxCenter[1] != 0) ||
		   (_bboxCenter[2] != 0))
			fp.println(buffer.toString() + " bboxCenter " +
						_bboxCenter[0] + " " +
						_bboxCenter[1] + " " +
						_bboxCenter[2]);

		if((_bboxSize[0] != -1) || (_bboxSize[1] != -1) ||
		   (_bboxSize[2] != -1))
			fp.println(buffer.toString() + " bboxSize " +
						_bboxSize[0] + " " +
						_bboxSize[1] + " " +
						_bboxSize[2]);

		fp.println(buffer.toString() + "}");
	}


	public void set_center(float[] center)
	{
		_center = center;
		if(have_browser)
			_set_center.setValue(center);
	}

	public float[] get_center()
	{
		return(_center);
	}

	public void set_rotation(float[] rotation)
	{
		_rotation = rotation;
		if(have_browser)
			_set_center.setValue(rotation);
	}

	public float[] get_rotation()
	{
		return(_rotation);
	}

	public void set_scale(float[] scale)
	{
		_scale = scale;
		if(have_browser)
			_set_scale.setValue(scale);
	}

	public float[] get_scale()
	{
		return(_scale);
	}

	public void set_scaleOrientation(float[] val)
	{
		_scaleOrientation = val;
		if(have_browser)
			_set_scaleOrientation.setValue(val);
	}

	public float[] get_scaleOrientation()
	{
		return(_scaleOrientation);
	}

	public void set_translation(float[] val)
	{
		_translation = val;

		System.out.println("The new tx is " + val[0] + " " + val[1] + " " + val[2]);
		if(have_browser)
			_set_translation.setValue(val);
	}

	public float[] get_translation()
	{
		return(_translation);
	}

}
