// Stores and emits vertices, lines, polygons

// Written by Bernie Roehl, November 1996

package lsystem;

import java.util.*;

import utility.*;

public class Emitter {
   protected Vector vertices = new Vector();
   protected Vector polygons = new Vector();
   protected boolean double_sided = false;

   public Emitter(boolean dsided) { double_sided = dsided; }

   public int addVertex(float x, float y, float z) {
       vertices.addElement(new Vec3(x, y, z));
       return vertices.size()-1;
   }

   public void addPolygon(int points[], int npoints, int color) {
       polygons.addElement(new Polygon(npoints, points, color));
   }

   public void addLine(int n1, int n2, int color) {
       int[] pts = new int[2];
       pts[0] = n1;  pts[1] = n2;
       polygons.addElement(new Polygon(2, pts, color));
   }

   public void emitPLG() {
       System.out.println("object " + vertices.size() + " " + polygons.size());
       Enumeration ev = vertices.elements();
       while (ev.hasMoreElements())
           System.out.println((Vec3) ev.nextElement());
       Enumeration ep = polygons.elements();
       while (ep.hasMoreElements()) {
           Polygon p = (Polygon) ep.nextElement();
           int n = p.getNumberOfPoints();
           System.out.print((0x8000 + p.getColor()) + " " + n);
           for (int j = 0; j < n; ++j)
               System.out.print(" " + p.getPoint(j));
           System.out.println();
           if (double_sided) {
               System.out.print((0x8000 + p.getColor()) + " " + n);
               for (int j = n-1; j >= 0; --j)
                   System.out.print(" " + p.getPoint(j));
               System.out.println();
           }
       }
   }

   public void emitVRML1() {
       System.out.println("#VRML V1.0 ascii");
       System.out.println("Coordinate3 {");
       System.out.println("\tpoint [");
       Enumeration ev = vertices.elements();
       while (ev.hasMoreElements())
           System.out.println("\t\t" + (Vec3) ev.nextElement());
       System.out.println("\t]");
       System.out.println("}");
   }

   public String getVertices() {
       StringBuffer buff = new StringBuffer(256);
       Enumeration en = vertices.elements();
       while (en.hasMoreElements()) {
           Vec3 v = (Vec3) en.nextElement();
           buff.append(" " + v.getX() + " " + v.getY() + " " + v.getZ() + "\n");
       }
       return buff.toString();
   }

   public int[] getFaces() {
       Vector points = new Vector();
       Enumeration en = polygons.elements();
       while (en.hasMoreElements()) {
           Polygon p = (Polygon) en.nextElement();
           if (p.getNumberOfPoints() < 3) continue;
           for (int i = 0; i < p.getNumberOfPoints(); ++i)
               points.addElement(new Integer(p.getPoint(i)));
           points.addElement(new Integer(-1));
       }
       int n = points.size();
       int pts[] = new int[n];
       for (int i = 0; i < n; ++i)
           pts[i] = ((Integer) points.elementAt(i)).intValue();
       return pts;
   }

   public int[] getLines() {
       Vector points = new Vector();
       Enumeration en = polygons.elements();
       while (en.hasMoreElements()) {
           Polygon p = (Polygon) en.nextElement();
           if (p.getNumberOfPoints() != 2) continue;
           for (int i = 0; i < p.getNumberOfPoints(); ++i)
               points.addElement(new Integer(p.getPoint(i)));
           points.addElement(new Integer(-1));
       }
       int n = points.size();
       int pts[] = new int[n];
       for (int i = 0; i < n; ++i)
           pts[i] = ((Integer) points.elementAt(i)).intValue();
       return pts;
   }

}

