// Lindenmayer System node

// Written by Bernie Roehl, November 1996

package lsystem;

import vrml.*;
import vrml.field.*;
import vrml.node.*;
import java.util.*;
import java.io.*;

public class LSystem extends Script  {

    public void initialize() {
        int ngenerations = ((SFInt32) getField("numberOfGenerations")).getValue();
        float angle = ((SFFloat) getField("initialAngle")).getValue();
        String axiom = ((SFString) getField("axiom")).getValue();
        MFString productionListField = (MFString) getField("productions");
        String[] productionList = new String[productionListField.getSize()];
        productionListField.getValue(productionList);
        String[] productions = new String[256];  // one for each ascii code
        boolean debug = ((SFBool) getField("debug")).getValue();

        for (int i = 0; i < productionList.length; ++i)
                productions[0xFF & (int) productionList[i].charAt(0)] = productionList[i].substring(2);

        for (int i = 0; i < ngenerations; ++i)
            axiom = process(axiom, productions);

        if (debug)
            System.out.println("Final axiom = " + axiom);

        Stack state_stack = new Stack();
        State current_state = new State();
        Emitter emitter = new Emitter(false);

        for (int i = 0; i < axiom.length(); ++i) {
            char c = axiom.charAt(i);
            switch (c) {
                case '[':
                    state_stack.push(current_state);
                    current_state = new State();
                    break;
                case ']':
                    current_state = (State) state_stack.pop();
                    break;
                default:
                    current_state.cmd(c, emitter);
                    break;
            }
        }
        try {
            SFNode coord_changed = (SFNode) getEventOut("coord_changed");
            coord_changed.setValue(getBrowser().createVrmlFromString(
                "Coordinate { point [\n"
                + emitter.getVertices()
                + " ] }\n")[0]);
        }
        catch (InvalidVRMLSyntaxException ex) {}

        MFInt32 coordIndex_changed = (MFInt32) getEventOut("coordIndex_changed");
        coordIndex_changed.setValue(emitter.getFaces());

        if (debug) {
            System.out.println("Vertices:\n" + emitter.getVertices());
            int[] p = emitter.getFaces();
            for (int i = 0; i < p.length; ++i)
                System.out.print(" " + p[i] + ((p[i] == -1) ? "\n" : ""));
        }

//        MFInt32 lineCoordIndex_changed = (MFInt32) getEventOut("lineCoordIndex_changed");
//        lineCoordIndex_changed.setValue(emitter.getLines());

    }

    static String process(String axiom, String[] productions) {
        StringBuffer result = new StringBuffer();
        for (int i = 0; i < axiom.length(); ++i) {
            char c = axiom.charAt(i);
            String s = productions[0xFF & (int) c];
            if (s == null)
                result.append(c);
            else
                result.append(s);
        }
        return new String(result);
    }

}

