// Quaternion routines

// Loosely based on Gavin Bell's rotation.c

public class Quaternion {

    float[] value = new float[4];

    public String toString() {
        return value[0] + " " + value[1] + " " + value[2] + " " + value[3];
    }

    public Quaternion(float[] q) {
        value[0] = q[0];  value[1] = q[1];
        value[2] = q[2];  value[3] = q[3]; 
    }

    public Quaternion(float[] axis, float angle) {
        float s = (float) Math.sin(angle/2);
        for (int i = 0; i < 3; ++i)
            value[i] = axis[i] * s;
        value[3] = (float) Math.cos(angle/2);
    }

    public Quaternion(char axis, float angle) {
        value[3] = (float) Math.cos(angle/2);
        value[0] = value[1] = value[2] = 0.0f;
        switch (axis) {
            case 'x': case 'X': value[0] = (float) Math.sin(angle/2); break;
            case 'y': case 'Y': value[1] = (float) Math.sin(angle/2); break;
            case 'z': case 'Z': value[2] = (float) Math.sin(angle/2); break;
        }
    }

    public Quaternion(float yaw, float pitch, float roll) {
        Quaternion q;
        if (yaw != 0) {
            q = new Quaternion('Y', yaw);
            if (pitch != 0)
                q.multiply(new Quaternion('X', pitch));
            if (roll != 0)
                q.multiply(new Quaternion('Z', roll));
        }
        else if (pitch != 0) {
            q = new Quaternion('X', pitch);
            if (roll != 0)
                q.multiply(new Quaternion('Z', roll));
        }
        else
            q = new Quaternion('Z', roll);
        value[0] = q.value[0];  value[1] = q.value[1];
        value[2] = q.value[2];  value[3] = q.value[3];
    }

    public float[] toAxisAndAngle() {
        float[] aa = new float[4];
        aa[3] = (float) Math.acos(value[3]) * 2;
        float s = (float) Math.sin(aa[3]/2);
        for (int i = 0; i < 3; ++i)
            aa[i] = value[i] / s;
        return aa;
    }

    void multiply(Quaternion q) {
        float[] tmp = new float[4];
        tmp[0] = q.value[3] * value[0] + q.value[0] * value[3]
               + q.value[1] * value[2] - q.value[2] * value[1];
        tmp[1] = q.value[3] * value[1] + q.value[1] * value[3]
               + q.value[2] * value[0] - q.value[0] * value[2];
        tmp[2] = q.value[3] * value[2] + q.value[2] * value[3]
               + q.value[0] * value[1] - q.value[1] * value[0];
        tmp[3] = q.value[3] * value[3] - q.value[0] * value[0]
               - q.value[1] * value[1] - q.value[2] * value[2];
        value[0] = tmp[0];  value[1] = tmp[1];
        value[2] = tmp[2];  value[3] = tmp[3];
    }

}

